`line 1 "F:\verilog\wb_z80\new_wb_z80\wb_z80\rtl\z80_testbed.v"






































































`timescale 1ns/10ps



module z80_testbed();

reg      rst, bist_req, clk;
wire     bist_ack;
wire     bist_err;






z80_core_top i_z80_core_top(                   
    .wb_dat_o(),      
    .wb_stb_o(),      
    .wb_cyc_o(),      
    .wb_we_o(),       
    .wb_adr_o(),      
    .wb_tga_o(),
    .wb_ack_i(1'b0),
    .wb_clk_i(clk),
    .wb_dat_i(8'h0),
    .wb_rst_i(rst),





 

    .int_req_i(1'b0)        
    );            


initial
begin
    clk = 0;
    

    #5000000     $finish;

    $display("simulation timeout"); 
end


always   #10 clk = ~clk;


initial
begin
    rst = 1'b0;
    bist_req = 1'b0;
    @( posedge clk)  rst = 1'b1;
    @( posedge clk)  rst = 1'b0;
    @( posedge clk)  bist_req = 1'b1;
    @( bist_ack ) ;
    @( posedge clk)
        if ( bist_err ) $display("TB bist error");
        else            $display( "TB bist ok"    );
    $finish;
end








endmodule

`line 1 "F:\verilog\wb_z80\new_wb_z80\wb_z80\rtl\z80_bist_logic.v"











































































module z80_bist_logic( bist_err_o, 
                       bist_ack_o,
                       wb_dat_o  ,
                       wb_ack_o  ,
                       int_req_o  ,
                       wb_adr_i  , 
                       wb_dat_i  , 
                       wb_we_i   , 
                       wb_cyc_i  ,
                       wb_stb_i  , 
                       wb_clk_i  ,
                       wb_tga_i  , 
                       int_req_i ,
                       wb_rst_i     );



output       bist_err_o;
output       bist_ack_o;
output [7:0] wb_dat_o;
output       wb_ack_o;
output       int_req_o;


input [15:0]  wb_adr_i;  
input         wb_we_i;   
input         wb_cyc_i;  
input         wb_stb_i;  
input [1:0]   wb_tga_i;
input         wb_clk_i;
input         wb_rst_i;  
input [7:0]   wb_dat_i;
input         int_req_i;





reg   [2:0]  bist_reg;
integer      i;
reg          wb_ack_o;
reg [7:0]    out_state;
reg [9:0]    int_count;
reg          int_req;
wire         int_ack;
wire         bist_int_en;


assign bist_err_o = bist_reg[1];
assign bist_ack_o = bist_reg[0];
assign bist_int_en = bist_reg[2];
wire    clk = wb_clk_i;
wire    rst = wb_rst_i;
assign  int_req_o = int_req_i | int_req & bist_int_en;




parameter   INT_OFFSET = 8'hfe;   
parameter   BIST_ADR = 16'hffff ; 
parameter   MY_IO_ADR = 8'h20 ;  


parameter   TAG_MEM   = 2'b00,
            TAG_IO    = 2'b01,   
            TAG_INT   = 2'b10;   







wire a2io = (wb_adr_i[7:0] == MY_IO_ADR) & wb_stb_i & wb_cyc_i & (wb_tga_i == TAG_IO);
wire a2bist = (wb_adr_i == BIST_ADR) & wb_stb_i & wb_cyc_i & (wb_tga_i == TAG_MEM);
assign int_ack = wb_stb_i & wb_cyc_i & (wb_tga_i == TAG_INT);


always @(posedge clk or posedge rst)
begin
    if (wb_rst_i )                                 wb_ack_o <= 1'b0;
    else if((a2io | a2bist | int_ack) & !wb_ack_o) wb_ack_o <= 1'b1;
    else                                           wb_ack_o <= 1'b0;    
end


always @(posedge clk or posedge rst)
    if (a2io & wb_we_i & wb_ack_o)  $write("%s",wb_dat_i); 







assign wb_dat_o = int_ack ? INT_OFFSET : out_state;

always @(posedge clk or posedge rst)
begin
    if (wb_rst_i)          out_state <=  8'h7f;
    else if (a2io & !wb_we_i & wb_ack_o)
        case (out_state)
            8'h7f:         out_state <=  8'h55 ;
            8'h55:         out_state <=  8'h80 ;
            8'h80:         out_state <=  8'h00 ;
            8'h00:         out_state <=  8'hff ;
            8'hff:         out_state <=  8'haa ;
            8'haa:         out_state <=  8'h7f ;
            default:       out_state <=  8'h7f ;
        endcase
end






wire wb_wr = wb_cyc_i & wb_stb_i & wb_we_i;
wire my_adr = (wb_tga_i == 2'b00) & ( wb_adr_i == BIST_ADR);

always @(posedge wb_clk_i or wb_rst_i) 
    if (wb_rst_i)                         bist_reg <= 3'b0;
    else if (my_adr & wb_wr & wb_ack_o)   bist_reg <= wb_dat_i[2:0];


initial 
begin
    $display("BL messages from Bist logic  TB messages from test bench  - others from test" );
    $display("BL dump a few memory locations to be sure initialization is sane") ;
    $readmemh( "readmem.txt", z80_testbed.i_z80_core_top.i_z80_sram.mem );
    
    for (i=0; i<10; i=i+1)
        $display( "BL mem [%0d] = %h", i, z80_testbed.i_z80_core_top.i_z80_sram.mem[i]); 
end



always @(posedge wb_clk_i or wb_rst_i) 
    if (wb_rst_i)              int_count <=10'h0;
    else                       int_count <= int_count + 10'h1;

always @(posedge wb_clk_i or wb_rst_i)
    if (wb_rst_i)                 int_req <= 1'b0;
    else if (int_count==10'h3ff)  int_req <= 1'b1;
    else if ( int_ack )           int_req <= 1'b0;
endmodule

`line 1 "F:\verilog\wb_z80\new_wb_z80\wb_z80\rtl\z80_core_top.v"


















































































module z80_core_top(  
                      wb_dat_o,      
                      wb_stb_o,      
                      wb_cyc_o,      
                      wb_we_o,       
                      wb_adr_o,      
                      wb_tga_o,
                      wb_ack_i,
                      wb_clk_i,
                      wb_dat_i,
                      wb_rst_i,





                      int_req_i 

);



output [7:0]    wb_dat_o;
output          wb_stb_o;
output          wb_cyc_o;
output          wb_we_o;
output [15:0]   wb_adr_o;
output [1:0]    wb_tga_o;




input           wb_ack_i;
input           wb_clk_i;
input  [7:0]    wb_dat_i;
input           wb_rst_i;
input           int_req_i;











wire   [15:0]    wb_adr_o; 
wire   [9:0]     ir1, ir2;
wire   [15:0]    nn;
wire   [15:0]    sp;
wire   [7:0]     ar, fr, br, cr, dr, er, hr, lr, intr;
wire   [15:0]    ixr, iyr;
wire   [7:0]     wb_dat_i, wb_dat_o, sdram_do, cfg_do, bist_do;
wire   [15:0]    add16;     
wire   [15:0]    adr_alu;   
wire   [7:0]     alu8_out, sh_alu, bit_alu;  
                                             

wire          sram_addr;
wire          ce_sram;  
wire    [7:0] wb_rd_dat;
wire          wb_ack; 



























wire bist_io_ack = 1'b0; 
wire [7:0] bist_dat_o = 8'b0;




z80_memstate2 i_z80_memstate2(
                .wb_adr_o(wb_adr_o), .wb_we_o(wb_we_o), .wb_cyc_o(wb_cyc_o), .wb_stb_o(wb_stb_o), .wb_tga_o(wb_tga_o), .wb_dat_o(wb_dat_o), 
                .exec_ir2(exec_ir2), 
                .exec_decbc(exec_decbc), .exec_decb(exec_decb), 
                .ir1(ir1), .ir2(ir2), .ir1dd(ir1dd), .ir1fd(ir1fd), .ir2dd(ir2dd), .ir2fd(ir2fd), .nn(nn), .sp(sp),
                .upd_ar(upd_ar), .upd_br(upd_br), .upd_cr(upd_cr), .upd_dr(upd_dr), .upd_er(upd_er), .upd_hr(upd_hr), .upd_lr(upd_lr),.upd_fr(upd_fr),
                .beq0(br_eq0), .ceq0(cr_eq0),
                .ar(ar), .fr(fr), .br(br), .cr(cr), .dr(dr), .er(er), .hr(hr), .lr(lr), 
                .ixr(ixr), .iyr(iyr), .intr(intr),
                .wb_dat_i(wb_rd_dat), .wb_ack_i(wb_ack), 
                .int_req_i(bist_int_req),
                .add16(add16),
                .alu8_out(alu8_out),
                .adr_alu(adr_alu),   
                .blk_mv_upd_hl(blk_mv_upd_hl),
                .blk_mv_upd_de(blk_mv_upd_de),
                .sh_alu(sh_alu),
                .bit_alu(bit_alu),
                .wb_clk_i(wb_clk_i),
                .rst_i(wb_rst_i)         
                 );


z80_inst_exec i_z80_inst_exec( 
                  .br_eq0(br_eq0),
                  .cr_eq0(cr_eq0),
                  .upd_ar(upd_ar), .upd_br(upd_br), .upd_cr(upd_cr), .upd_dr(upd_dr), .upd_er(upd_er), .upd_hr(upd_hr), .upd_lr(upd_lr),.upd_fr(upd_fr),
                  .ar(ar), .fr(fr), .br(br), .cr(cr), .dr(dr), .er(er), .hr(hr), .lr(lr), .intr(intr), 
                  .ixr(ixr), .iyr(iyr), .add16(add16), .alu8_out(alu8_out),
                  .adr_alu(adr_alu),   
                  .blk_mv_upd_hl(blk_mv_upd_hl),
                  .blk_mv_upd_de(blk_mv_upd_de),
                   .sh_alu(sh_alu),
                   .bit_alu(bit_alu),
                   .exec_ir2(exec_ir2),
                   .exec_decbc(exec_decbc), .exec_decb(exec_decb), 
                   .ir2(ir2),
                   .clk(wb_clk_i),
                   .rst(wb_rst_i),
                   .nn(nn), .sp(sp),
                   .ir2dd(ir2dd),
                   .ir2fd(ir2fd)
                   );








assign       sram_addr   =  (wb_tga_o == 2'b00);
assign       ce_sram     = sram_addr & wb_cyc_o & wb_stb_o;
assign       wb_rd_dat   =  sram_addr   ? sdram_do :
                            bist_io_ack ? bist_dat_o :
                                          wb_dat_i;
assign       wb_ack = ce_sram | bist_io_ack | wb_ack_i;



z80_sram #(16) i_z80_sram(
    
    .clk(wb_clk_i), .rst(wb_rst_i), .ce(ce_sram), .we(wb_we_o), .oe(1'b1), 
    .addr(wb_adr_o[15:0]), .di(wb_dat_o), .do(sdram_do)
    );








endmodule

`line 1 "F:\verilog\wb_z80\new_wb_z80\wb_z80\rtl\z80_inst_exec.v"














































































































module z80_inst_exec( br_eq0,
                  cr_eq0,
                  upd_ar, upd_br, upd_cr, upd_dr, upd_er, upd_hr, upd_lr,upd_fr,
                  ar, fr, br, cr, dr, er, hr, lr, intr,  
                  ixr, iyr, add16, alu8_out,  sh_alu, bit_alu,
                   exec_ir2,
                   exec_decbc, exec_decb,
                   adr_alu,
                   blk_mv_upd_hl,
                   blk_mv_upd_de,
                   ir2,
                   clk,
                   rst,
                   nn, sp,
                   ir2dd,
                   ir2fd
                   );


output          br_eq0;
output          cr_eq0;
output          upd_ar, upd_br, upd_cr, upd_dr, upd_er, upd_hr, upd_lr,upd_fr;
output  [7:0]   ar, fr, br, cr, dr, er, hr, lr, intr; 
output  [15:0]  ixr, iyr;
output  [15:0]  add16;
output  [7:0]   alu8_out;   
                            
output  [7:0]   sh_alu;
output  [7:0]   bit_alu;

input        exec_ir2;
input        exec_decbc;  
input        exec_decb;   
input [9:0]  ir2;
input        clk;
input        rst;
input [15:0] nn, sp;
input        ir2dd;       
input        ir2fd;
input [15:0] adr_alu;
input        blk_mv_upd_hl;
input        blk_mv_upd_de;

`line 1 "F:\verilog\wb_z80\new_wb_z80\wb_z80\rtl\opcodes.v"































































































parameter  NOP          = 10'h00,
           LDsBC_NN     = 10'h01,
           LDs6BC7_A    = 10'h02,
           INCsBC       = 10'h03,
           INCsB        = 10'h04,
           DECsB        = 10'h05,
           LDsB_N       = 10'h06,
           RLCA         = 10'h07,
           EXsAF_AFp    = 10'h08,
           ADDsHL_BC    = 10'h09,
           LDsA_6BC7    = 10'h0A,
           DECsBC       = 10'h0B,
           INCsC        = 10'h0C,
           DECsC        = 10'h0D,
           LDsC_N       = 10'h0E,
           RRCA         = 10'h0F,
           DJNZs$t2     = 10'h10,
           LDsDE_NN     = 10'h11,
           LDs6DE7_A    = 10'h12,
           INCsDE       = 10'h13,
           INCsD        = 10'h14,
           DECsD        = 10'h15,
           LDsD_N       = 10'h16,
           RLA          = 10'h17,
           JRs$t2       = 10'h18,
           ADDsHL_DE    = 10'h19,
           LDsA_6DE7    = 10'h1A,
           DECsDE       = 10'h1B,
           INCsE        = 10'h1C,
           DECsE        = 10'h1D,
           LDsE_N       = 10'h1E,
           RRA          = 10'h1F,
           JRsNZ_$t2    = 10'h20,
           LDsHL_NN     = 10'h21,
           LDs6NN7_HL   = 10'h22,
           INCsHL       = 10'h23,
           INCsH        = 10'h24,
           DECsH        = 10'h25,
           LDsH_N       = 10'h26,
           DAA          = 10'h27,
           JRsZ_$t2     = 10'h28,
           ADDsHL_HL    = 10'h29,
           LDsHL_6NN7   = 10'h2A,
           DECsHL       = 10'h2B,
           INCsL        = 10'h2C,
           DECsL        = 10'h2D,
           LDsL_N       = 10'h2E,
           CPL          = 10'h2F,
           JRsNC_$t2    = 10'h30,
           LDsSP_NN     = 10'h31,
           LDs6NN7_A    = 10'h32,
           INCsSP       = 10'h33,
           INCs6HL7     = 10'h34,
           DECs6HL7     = 10'h35,
           LDs6HL7_N    = 10'h36,
           SCF          = 10'h37,
           JRsC_$t2     = 10'h38,
           ADDsHL_SP    = 10'h39,
           LDsA_6NN7    = 10'h3A,
           DECsSP       = 10'h3B,
           INCsA        = 10'h3C,
           DECsA        = 10'h3D,
           LDsA_N       = 10'h3E,
           CCF          = 10'h3F,
           LDsB_B       = 10'h40,
           LDsB_C       = 10'h41,
           LDsB_D       = 10'h42,
           LDsB_E       = 10'h43,
           LDsB_H       = 10'h44,
           LDsB_L       = 10'h45,
           LDsB_6HL7    = 10'h46,
           LDsB_A       = 10'h47,
           LDsC_B       = 10'h48,
           LDsC_C       = 10'h49,
           LDsC_D       = 10'h4A,
           LDsC_E       = 10'h4B,
           LDsC_H       = 10'h4C,
           LDsC_L       = 10'h4D,
           LDsC_6HL7    = 10'h4E,
           LDsC_A       = 10'h4F,
           LDsD_B       = 10'h50,
           LDsD_C       = 10'h51,
           LDsD_D       = 10'h52,
           LDsD_E       = 10'h53,
           LDsD_H       = 10'h54,
           LDsD_L       = 10'h55,
           LDsD_6HL7    = 10'h56,
           LDsD_A       = 10'h57,
           LDsE_B       = 10'h58,
           LDsE_C       = 10'h59,
           LDsE_D       = 10'h5A,
           LDsE_E       = 10'h5B,
           LDsE_H       = 10'h5C,
           LDsE_L       = 10'h5D,
           LDsE_6HL7    = 10'h5E,
           LDsE_A       = 10'h5F,
           LDsH_B       = 10'h60,
           LDsH_C       = 10'h61,
           LDsH_D       = 10'h62,
           LDsH_E       = 10'h63,
           LDsH_H       = 10'h64,
           LDsH_L       = 10'h65,
           LDsH_6HL7    = 10'h66,
           LDsH_A       = 10'h67,
           LDsL_B       = 10'h68,
           LDsL_C       = 10'h69,
           LDsL_D       = 10'h6A,
           LDsL_E       = 10'h6B,
           LDsL_H       = 10'h6C,
           LDsL_L       = 10'h6D,
           LDsL_6HL7    = 10'h6E,
           LDsL_A       = 10'h6F,
           LDs6HL7_B    = 10'h70,
           LDs6HL7_C    = 10'h71,
           LDs6HL7_D    = 10'h72,
           LDs6HL7_E    = 10'h73,
           LDs6HL7_H    = 10'h74,
           LDs6HL7_L    = 10'h75,
           HALT         = 10'h76,
           LDs6HL7_A    = 10'h77,
           LDsA_B       = 10'h78,
           LDsA_C       = 10'h79,
           LDsA_D       = 10'h7A,
           LDsA_E       = 10'h7B,
           LDsA_H       = 10'h7C,
           LDsA_L       = 10'h7D,
           LDsA_6HL7    = 10'h7E,
           LDsA_A       = 10'h7F,
           ADDsA_B      = 10'h80,
           ADDsA_C      = 10'h81,
           ADDsA_D      = 10'h82,
           ADDsA_E      = 10'h83,
           ADDsA_H      = 10'h84,
           ADDsA_L      = 10'h85,
           ADDsA_6HL7   = 10'h86,
           ADDsA_A      = 10'h87,
           ADCsA_B      = 10'h88,
           ADCsA_C      = 10'h89,
           ADCsA_D      = 10'h8A,
           ADCsA_E      = 10'h8B,
           ADCsA_H      = 10'h8C,
           ADCsA_L      = 10'h8D,
           ADCsA_6HL7   = 10'h8E,
           ADCsA_A      = 10'h8F,
           SUBsB        = 10'h90,
           SUBsC        = 10'h91,
           SUBsD        = 10'h92,
           SUBsE        = 10'h93,
           SUBsH        = 10'h94,
           SUBsL        = 10'h95,
           SUBs6HL7     = 10'h96,
           SUBsA        = 10'h97,
           SBCsB        = 10'h98,
           SBCsC        = 10'h99,
           SBCsD        = 10'h9A,
           SBCsE        = 10'h9B,
           SBCsH        = 10'h9C,
           SBCsL        = 10'h9D,
           SBCs6HL7     = 10'h9E,
           SBCsA        = 10'h9F,
           ANDsB        = 10'hA0,
           ANDsC        = 10'hA1,
           ANDsD        = 10'hA2,
           ANDsE        = 10'hA3,
           ANDsH        = 10'hA4,
           ANDsL        = 10'hA5,
           ANDs6HL7     = 10'hA6,
           ANDsA        = 10'hA7,
           XORsB        = 10'hA8,
           XORsC        = 10'hA9,
           XORsD        = 10'hAA,
           XORsE        = 10'hAB,
           XORsH        = 10'hAC,
           XORsL        = 10'hAD,
           XORs6HL7     = 10'hAE,
           XORsA        = 10'hAF,
           ORsB         = 10'hB0,
           ORsC         = 10'hB1,
           ORsD         = 10'hB2,
           ORsE         = 10'hB3,
           ORsH         = 10'hB4,
           ORsL         = 10'hB5,
           ORs6HL7      = 10'hB6,
           ORsA         = 10'hB7,
           CPsB         = 10'hB8,
           CPsC         = 10'hB9,
           CPsD         = 10'hBA,
           CPsE         = 10'hBB,
           CPsH         = 10'hBC,
           CPsL         = 10'hBD,
           CPs6HL7      = 10'hBE,
           CPsA         = 10'hBF,
           RETsNZ       = 10'hC0,
           POPsBC       = 10'hC1,
           JPsNZ        = 10'hC2,
           JP           = 10'hC3,
           CALLsNZ_NN   = 10'hC4,
           PUSHsBC      = 10'hC5,
           ADDsA_N      = 10'hC6,
           RSTs0        = 10'hC7,
           RETsZ        = 10'hC8,
           RET          = 10'hC9,
           JPsZ         = 10'hCA,
           CALLsZ_NN    = 10'hCC,
           CBgrp        = 10'hCB,
           CALLsNN      = 10'hCD,
           ADCsA_N      = 10'hCE,
           RSTs8H       = 10'hCF,
           RETsNC       = 10'hD0,
           POPsDE       = 10'hD1,
           JPsNC        = 10'hD2,
           OUTs6N7_A    = 10'hD3,
           CALLsNC_NN   = 10'hD4,
           PUSHsDE      = 10'hD5,
           SUBsN        = 10'hD6,
           RSTs10H      = 10'hD7,
           RETsC        = 10'hD8,
           EXX          = 10'hD9,
           JPsC         = 10'hDA,
           INsA_6N7     = 10'hDB,
           CALLsC_NN    = 10'hDC,
           DDgrp        = 10'hDD,
           SBCsA_N      = 10'hDE,
           RSTs18H      = 10'hDF,
           RETsPO       = 10'hE0,
           POPsHL       = 10'hE1,
           JPsPO        = 10'hE2,
           EXs6SP7_HL   = 10'hE3,
           CALLsPO_NN   = 10'hE4,
           PUSHsHL      = 10'hE5,
           ANDsN        = 10'hE6,
           RSTs20H      = 10'hE7,
           RETsPE       = 10'hE8,
           JPsHL        = 10'hE9,
           JPsPE        = 10'hEA,
           EXsDE_HL     = 10'hEB,
           CALLsPE_NN   = 10'hEC,
           EDgrp        = 10'hED,
           XORsN        = 10'hEE,
           RSTs28H      = 10'hEF,
           RETsP        = 10'hF0,
           POPsAF       = 10'hF1,
           JPsP         = 10'hF2,
           DI           = 10'hF3,
           CALLsP_NN    = 10'hF4,
           PUSHsAF      = 10'hF5,
           ORsN         = 10'hF6,
           RSTs30H      = 10'hF7,
           RETsM        = 10'hF8,
           LDsSP_HL     = 10'hF9,
           JPsM         = 10'hFA,
           EI           = 10'hFB,
           CALLsM_NN    = 10'hFC,
           FDgrp        = 10'hFD,
           CPsN         = 10'hFE,
           RSTs38H      = 10'hFF,




           CB_RLC   = 7'b01_00_000,  
           CB_RRC   = 7'b01_00_001,  
           CB_RL    = 7'b01_00_010,  
           CB_RR    = 7'b01_00_011,  
           CB_SLA   = 7'b01_00_100,  
           CB_SRA   = 7'b01_00_101,  
           CB_SLL   = 7'b01_00_110,  
           CB_SRL   = 7'b01_00_111,  
           
           CB_BIT   = 4'b01_01,    
           CB_RES   = 4'b01_10,    
           CB_SET   = 4'b01_11,    
           
           CB_MEM   = 3'b110,   
                             
                        





           ED_IMs0      =  10'h246,  
           ED_LDsI_A    =  10'h247,  
           ED_IMs1      =  10'h256,  
           ED_LDsA_I    =  10'h257,  
           ED_IMs2      =  10'h25E,  
           ED_RRD       =  10'h267,  
           ED_RLD       =  10'h26F,  
                                    
           
           ED_LDI       =  10'h2A0,  
           ED_CPI       =  10'h2A1,  
           ED_INI       =  10'h2A2,  
           ED_OUTI      =  10'h2A3,  
           ED_LDD       =  10'h2A8,  
           ED_CPD       =  10'h2A9,  
           ED_IND       =  10'h2AA,  
           ED_OUTD      =  10'h2AB,  
           ED_LDIR      =  10'h2B0,  
           ED_CPIR      =  10'h2B1,  
           ED_INIR      =  10'h2B2,  
           ED_OTIR      =  10'h2B3,  
           ED_LDDR      =  10'h2B8,  
           ED_CPDR      =  10'h2B9,  
           ED_INDR      =  10'h2BA,  
           ED_OTDR      =  10'h2BB,  





























        ED_INsREG_6C7  =    7'b1001___000, 
        ED_OUTs6C7_REG =    7'b1001___001, 
        ED_SBCsHL_REG  =    8'b1001__0010, 
        ED_ADCsHL_REG  =    8'b1001__1010, 
        ED_LDs6NN7_REG =    8'b1001__0011, 
        ED_LDsREG_6NN7 =    8'b1001__1011, 
        ED_NEG         =    7'b1001___100, 
        ED_RETN        =    7'b1001___101, 
        ED_RETI        =  10'h24D, 
        
        DBL_REG_BC   = 2'b00,  
        DBL_REG_DE   = 2'b01,  
        DBL_REG_HL   = 2'b10,  
        DBL_REG_SP   = 2'b11,  

        REG8_B = 3'b000,
        REG8_C = 3'b001,
        REG8_D = 3'b010,
        REG8_E = 3'b011,
        REG8_H = 3'b100,
        REG8_L = 3'b101,
        REG8_MEM = 3'b110,
        REG8_A = 3'b111;

































 







 







          
          
`line 154 "F:\verilog\wb_z80\new_wb_z80\wb_z80\rtl\z80_inst_exec.v"





wire [7:0]   src_hr ;
wire [7:0]   src_lr ;




wire   [15:0]   src_a, src_b;
wire   [15:0]   add16;
wire            sf, zf, f5f, hf, f3f, pvf, nf, cf; 
wire   [7:0]    daa_alu; 
wire            daa_cry;
wire            upd_ar, upd_br, upd_cr, upd_dr, upd_er, upd_fr, upd_hr, upd_lr;
wire        c_8out3;
wire [7:0]  add_8bit;

wire  [15:0]  src_dblhr       ;

wire  [7:0]  src_pqr20       ;
wire  [7:0]   src_pqr53       ;
wire  [15:0]  src_dbl         ;
wire  [7:0]   alu8_fr         ;
wire          alu8_nf         ;
wire          c_8out7         ;
wire          alu8_cry        ;
wire          alu8_pvf        ;
wire          alu8_hcry       ;
wire  [7:0]   alu8_out        ;
wire          add16_ofl       ;
wire          c_16out7        ;
wire          c_16out11       ;
wire          c_16out15       ;
wire          c_16in0         ;
wire          sh_cry          ;
wire  [7:0]   sh_alu          ;
wire          sh_alu_act      ;
wire          bit_alu_act     ;
wire  [7:0]   bit_alu         ;
wire  [7:0]   decc_alu        ;
wire  [7:0]   decb_alu        ;
wire          upd_a_alu8      ;
wire          up_a_sh_alu     ;
wire          up_a_src_pqr    ;
wire          up_a_n          ;
wire          upd_b_alu8      ;
wire          up_b_src_pqr    ;
wire          up_b_add16      ;
wire [7:0]    sh_src          ;    

wire          up_c_add16       ;
wire          upd_c_alu8       ;
wire          up_c_src_pqr     ;
wire          up_d_add16       ;
wire          upd_d_alu8       ;
wire          up_d_src_pqr     ;
wire          up_e_add16       ;
wire          upd_e_alu8       ;
wire          up_e_src_pqr     ;
wire          up_h_add16       ;
wire          upd_h_alu8       ;
wire          upd_h_src_pqr    ;
wire          up_l_add16       ;
wire          upd_l_alu8       ;
wire          upd_l_src_pqr    ;
wire          upd_bc_cpi       ;                      
wire          upd_fr_alu8      ;
wire          upd_fr_add16     ;
wire          upd_fr_edadd16   ;
wire          upd_fr_sh        ;
wire          upd_fr_cbsh      ;

wire          ed_blk_cp        ;  
wire          c_8in0           ;



reg [7:0] ar, fr, br, cr, dr, er, hr, lr, intr;
reg [7:0] ap, fp, bp, cp, dp, ep, hp, lp; 
reg [15:0] ixr, iyr;








assign  src_hr = ir2dd ? ixr[15:8] : 
                 ir2fd ? iyr[15:8] :
                 hr                   ;
                 
assign  src_lr = ir2dd ? ixr[7:0] : 
                 ir2fd ? iyr[7:0] :
                 lr                 ;

assign src_dblhr = ir2dd ? ixr :    
                   ir2fd ? iyr :    
                   {hr, lr}      ;




assign  br_eq0 = ~|br; 
assign  cr_eq0 = ~|cr; 
assign  src_pqr20 = {8{ir2[2:0]==REG8_B   }} & br     |
                    {8{ir2[2:0]==REG8_C   }} & cr     |
                    {8{ir2[2:0]==REG8_D   }} & dr     |
                    {8{ir2[2:0]==REG8_E   }} & er     |
                    {8{ir2[2:0]==REG8_H   }} & src_hr |
                    {8{ir2[2:0]==REG8_L   }} & src_lr |
                    {8{ir2[2:0]==REG8_MEM}} & nn[15:8] |
                    {8{ir2[2:0]==REG8_A   }} & ar      ;
                    
assign src_pqr53 =  {8{ir2[5:3]==REG8_B   }} & br     |
                    {8{ir2[5:3]==REG8_C   }} & cr     |
                    {8{ir2[5:3]==REG8_D   }} & dr     |
                    {8{ir2[5:3]==REG8_E   }} & er     |
                    {8{ir2[5:3]==REG8_H   }} & src_hr |
                    {8{ir2[5:3]==REG8_L   }} & src_lr |
                    {8{ir2[5:3]==REG8_MEM}} & nn[15:8] |
                    {8{ir2[5:3]==REG8_A   }} & ar      ;


assign src_dbl =   {16{ir2[5:4]==2'b00}} & {br, cr}  |
                   {16{ir2[5:4]==2'b01}} & {dr, er}  |
                   {16{ir2[5:4]==2'b10}} & src_dblhr |   
                   {16{ir2[5:4]==2'b11}} & sp         ;

assign sh_src =   ir2[8] & ir2dd ?  nn[15:8]   :
                  ir2[8] & ir2fd ?  nn[15:8]   :
                  ir2[8]          ?  src_pqr20  :
                                     ar          ;

































wire [7:0]  src_pqri;  
wire [7:0]  src_aor_cnst = ed_blk_cp ?  ar    :  
                              ir2[9] ?  8'h0  :  
                              ir2[7] ?  ar    :
                              ir2[0] ?  8'hff :
                                        8'h00  ;
                                    


assign alu8_fr ={alu8_out[7], ~|alu8_out, alu8_out[5], alu8_hcry,

                 alu8_out[3], alu8_pvf, alu8_nf,  alu8_cry };
 
assign alu8_pvf = ir2[7] & (ir2[5:3]==7'b100 | ir2[5:3]==7'b101 | ir2[5:3]==7'b110) ?
                                                                    ~^alu8_out   : 
                (src_aor_cnst[7]==src_pqri[7]) & (src_aor_cnst[7]!=alu8_out[7])  ; 

assign alu8_nf = (ir2[7:3]==5'b10010)       | 
                 (ir2[7:3]==5'b10011)       | 
                 (ir2[7:6]==2'b00) & ir2[0] | 
                 ir2[9]                      ;

assign {c_8out3, add_8bit[3:0]} = {1'b0, src_aor_cnst[3:0]} + {1'b0, src_pqri[3:0]}   + {4'b0, c_8in0};



assign {c_8out7, add_8bit[7:4]} = {1'b0, src_aor_cnst[7:4]} + {1'b0, src_pqri[7:4]}   + {4'b0, c_8out3};






assign {alu8_cry, alu8_hcry, alu8_out,  src_pqri, c_8in0 }=
   
   ed_blk_cp ?                         {c_8out7,c_8out3,  add_8bit,   ~nn[15:8], 1'h1} :   

   {19{ir2[7] & ir2[5:3]==3'b000}} & ({c_8out7,c_8out3,  add_8bit,    src_pqr20, 1'b0} )  |
   {19{ir2[7] & ir2[5:3]==5'b001}} & ({c_8out7,c_8out3,  add_8bit,    src_pqr20,   cf} )  |
   {19{ir2[7] & ir2[5:3]==5'b010}} & ({~c_8out7,c_8out3,  add_8bit,   ~src_pqr20, 1'h1} )  |
   {19{ir2[7] & ir2[5:3]==5'b011}} & ({~c_8out7,c_8out3,  add_8bit,   ~src_pqr20, ~cf } )  |
   {19{ir2[7] & ir2[5:3]==5'b100}} & ({1'b0   ,1'b1   , ar & src_pqr20, src_pqr20, 1'b0} )|
   {19{ir2[7] & ir2[5:3]==5'b101}} & ({1'b0   ,1'b0   , ar ^ src_pqr20, src_pqr20, 1'b0} )|
   {19{ir2[7] & ir2[5:3]==5'b110}} & ({1'b0   ,1'b0   , ar | src_pqr20, src_pqr20, 1'b0} )|
   {19{ir2[7] & ir2[5:3]==5'b111}} & ({~c_8out7,c_8out3,  add_8bit,   ~src_pqr20,  1'h1})  |
   {19{(ir2[7:6]==2'b00)& ~ir2[0] }}& ({     cf,c_8out3,  add_8bit,    src_pqr53,  1'h1}) |
   {19{(ir2[7:6]==2'b00)&  ir2[0] }}& ({     cf,c_8out3,  add_8bit,    src_pqr53,  1'h0}) |
   {19{(ir2[7:6]==2'b01)          }}& ({~c_8out7,c_8out3,  add_8bit,          ~ar,  1'h1})  ;














assign add16_ofl = (src_a[15] == src_b[15]) & (src_a[15] != add16[15]);
 
assign {c_16out7,  add16[7:0]}  = {1'b0, src_a[7:0]}   + {1'b0, src_b[7:0]   } + {8'b0, c_16in0};
assign {c_16out11, add16[11:8]} = {1'b0, src_a[11:8]}  + {1'b0, src_b[11:8]  } + {4'b0, c_16out7};
assign {c_16out15, add16[15:12]} = {1'b0, src_a[15:12]} + {1'b0, src_b[15:12]} + {4'b0, c_16out11};

assign  { src_a,     src_b, c_16in0} =         
   {33{ir2[3:0] == 4'h9}} & {src_dblhr, src_dbl  ,1'b0 }   |  
   {33{ir2[3:0] == 4'hb}} & {16'hffff , src_dbl  ,1'b0 }   |  
   {33{ir2[3:0] == 4'h3}} & {16'h0001 , src_dbl  ,1'b0 }   |  
   {33{ir2[3:0] == 4'h2}} & {src_dblhr, ~src_dbl , ~cf }   |  
   {33{ir2[3:0] == 4'ha}} & {src_dblhr, src_dbl  , cf  }    ; 
                          











assign {sh_cry, sh_alu} =  {9{ir2[5:3]==3'b000}} & {sh_src, sh_src[7] }                 | 
                           {9{ir2[5:3]==3'b001}} & {sh_src[0], sh_src[0], sh_src[7:1]}  | 
                           {9{ir2[5:3]==3'b010}} & {sh_src, cf  }                       | 
                           {9{ir2[5:3]==3'b011}} & {sh_src[0], cf, sh_src[7:1] }        | 
                           {9{ir2[5:3]==3'b100}} & {sh_src, 1'b0}                       |  
                           {9{ir2[5:3]==3'b101}} & {sh_src[0], sh_src[7], sh_src[7:1]}  |  
                           {9{ir2[5:3]==3'b110}} & {sh_src, 1'b1}                       |  
                           {9{ir2[5:3]==3'b111}} & {sh_src[0], 1'b0, sh_src[7:1]}      ;   


 
 assign sh_alu_act = ir2[9:6] == 4'b0100;
 
 
 
 
 
 
 
 






assign bit_alu_act = ir2[9:6] == CB_RES |
                     ir2[9:6] == CB_SET ;

wire [7:0] bit_decode = {8{ir2[5:3] == 3'h0}} & 8'h01 |
                        {8{ir2[5:3] == 3'h1}} & 8'h02 |
                        {8{ir2[5:3] == 3'h2}} & 8'h04 |
                        {8{ir2[5:3] == 3'h3}} & 8'h08 |
                        {8{ir2[5:3] == 3'h4}} & 8'h10 |
                        {8{ir2[5:3] == 3'h5}} & 8'h20 |
                        {8{ir2[5:3] == 3'h6}} & 8'h40 |
                        {8{ir2[5:3] == 3'h7}} & 8'h80 ;

assign bit_alu = {8{ir2[9:6] == CB_BIT}} & ( sh_src & bit_decode)  |
                 {8{ir2[9:6] == CB_RES}} & ( sh_src & ~bit_decode) |
                 {8{ir2[9:6] == CB_SET}} & ( sh_src | bit_decode)   ;
                 




assign decc_alu  =  cr + 8'hff ;
assign decb_alu  =  br + ( exec_decb ? 8'hff :    
                              cr_eq0 ? 8'hff :    
                                      8'h00 );   










wire [3:0] ls_nbl   =  (!nf & hf)                 ?  4'h6:
                       (!nf & (ar[3:0] > 4'h9))   ?  4'h6:
                       (nf  & hf )                ?  4'ha:                              
                                                     4'h0;

wire [4:0] ms_nbl   =  (!nf & cf)                 ?  5'h16:    
                       (!nf & (ar[3:0]  > 4'h9))  ?  5'h16:
                       (!nf & (ar[3:0] == 4'h9) & 
                               (ar[3:0] > 4'h9))  ?  5'h16:
                       (nf  & !cf  &  hf )        ?  5'h0f:        
                       (nf  &  cf  & !hf )        ?  5'h1a:
                       (nf  &  cf  &  hf )        ?  5'h19:
                                                     5'h00;


assign {daa_cry, daa_alu} = { ms_nbl[4], {ar + { ms_nbl[3:0], ls_nbl}}  } ; 






assign upd_a_alu8 =
    ADDsA_B      == ir2 | SUBsB      == ir2 |  ANDsB      == ir2 | ORsB         == ir2   |
    ADDsA_C      == ir2 | SUBsC      == ir2 |  ANDsC      == ir2 | ORsC         == ir2   |
    ADDsA_D      == ir2 | SUBsD      == ir2 |  ANDsD      == ir2 | ORsD         == ir2   |
    ADDsA_E      == ir2 | SUBsE      == ir2 |  ANDsE      == ir2 | ORsE         == ir2   |
    ADDsA_H      == ir2 | SUBsH      == ir2 |  ANDsH      == ir2 | ORsH         == ir2   |
    ADDsA_L      == ir2 | SUBsL      == ir2 |  ANDsL      == ir2 | ORsL         == ir2   |
    ADDsA_6HL7   == ir2 | SUBs6HL7   == ir2 |  ANDs6HL7   == ir2 | ORs6HL7      == ir2   |
    ADDsA_A      == ir2 | SUBsA      == ir2 |  ANDsA      == ir2 | ORsA         == ir2   |
    ADCsA_B      == ir2 | SBCsB      == ir2 |  XORsB      == ir2 |
    ADCsA_C      == ir2 | SBCsC      == ir2 |  XORsC      == ir2 | INCsA        == ir2   |
    ADCsA_D      == ir2 | SBCsD      == ir2 |  XORsD      == ir2 | DECsA        == ir2   |
    ADCsA_E      == ir2 | SBCsE      == ir2 |  XORsE      == ir2 |
    ADCsA_H      == ir2 | SBCsH      == ir2 |  XORsH      == ir2 |
    ADCsA_L      == ir2 | SBCsL      == ir2 |  XORsL      == ir2 |
    ADCsA_6HL7   == ir2 | SBCs6HL7   == ir2 |  XORs6HL7   == ir2 |
    ADCsA_A      == ir2 | SBCsA      == ir2 |  XORsA      == ir2 |
    ADDsA_N      == ir2 | 
    ADCsA_N      == ir2 | 
    SUBsN        == ir2 | 
    SBCsA_N      == ir2 | 
    ANDsN        == ir2 | 
    XORsN        == ir2 | 
    ORsN         == ir2 ; 
assign up_a_sh_alu = 
    RLCA         == ir2   | 
    RRCA         == ir2   | 
    RRA          == ir2   | 
    RLA          == ir2   ; 
assign up_a_src_pqr = 
    LDsA_B       == ir2 |    
    LDsA_C       == ir2 |    
    LDsA_D       == ir2 |    
    LDsA_E       == ir2 |    
    LDsA_H       == ir2 |    
    LDsA_L       == ir2 |    
    LDsA_6HL7    == ir2 |    
    LDsA_A       == ir2 ;    
assign up_a_n =            
    LDsA_N       == ir2 | 
    LDsA_6BC7    == ir2 | 
    LDsA_6DE7    == ir2 | 
    LDsA_6NN7    == ir2 | 
    INsA_6N7     == ir2 | 
    (ED_INsREG_6C7 == {ir2[9:6],ir2[2:0]}) & (ir2[5:3] == REG8_A) ;














assign upd_ar = upd_a_alu8 | up_a_sh_alu | up_a_src_pqr | up_a_n |  ir2 == EXsAF_AFp |
                ir2 == EXX | ir2 == DAA  | ir2 == CPL   | ir2 == POPsAF | 
                ir2[2:0] == REG8_A & bit_alu_act | ir2[2:0] == REG8_A & sh_alu_act |
                ir2== ED_RRD | {ir2[9:6], ir2[2:0]} == ED_NEG | 
                ir2 == ED_LDsA_I ;
                
always @(posedge clk)
begin
    if (upd_a_alu8 & exec_ir2)       ar <= alu8_out;
    if (up_a_sh_alu & exec_ir2)      ar <= sh_alu; 
    if (up_a_src_pqr & exec_ir2)     ar <= src_pqr20;
    if (up_a_n  & exec_ir2)          ar <= nn[15:8];    
    if (ir2 == EXsAF_AFp & exec_ir2) ar <= ap; 
    if (ir2 == EXX & exec_ir2)       ar <= ap;
    if (ir2 == DAA & exec_ir2)       ar <= daa_alu;
    if (ir2 == CPL & exec_ir2)       ar <= ~ar;
    if (ir2 == POPsAF & exec_ir2)    ar <= nn[15:8];
    if (ir2[2:0] == REG8_A & 
             bit_alu_act & exec_ir2) ar <= bit_alu;
    if (ir2[2:0] == REG8_A & 
             sh_alu_act & exec_ir2)  ar <= sh_alu;
    if (ir2 == ED_RRD & exec_ir2) ar[3:0] <= nn[11:8];         
    if (ir2 == ED_RLD & exec_ir2) ar[3:0] <= nn[15:12];
    if ({ir2[9:6], ir2[2:0]} == ED_NEG & exec_ir2) ar <= alu8_out;  
    if (ir2 == ED_LDsA_I & exec_ir2) ar <= intr ;
end


























assign upd_b_alu8 =
    INCsB         == ir2 |
    DECsB         == ir2 ;


assign up_b_src_pqr =
    LDsB_B       == ir2 |
    LDsB_C       == ir2 |
    LDsB_D       == ir2 |
    LDsB_E       == ir2 |
    LDsB_H       == ir2 |
    LDsB_L       == ir2 |
    LDsB_6HL7    == ir2 |
    LDsB_A       == ir2 ;
assign up_b_add16 = 
    INCsBC       == ir2 |
    DECsBC       == ir2 ;











assign upd_bc_cpi = ed_blk_cp & exec_ir2;

assign upd_br = upd_b_alu8 | up_b_src_pqr | up_b_add16 | LDsBC_NN  == ir2 | 
                POPsBC    == ir2 | EXX       == ir2 | LDsB_N    == ir2    | 
                ir2[2:0] == REG8_B & bit_alu_act | ir2[2:0] == REG8_B & sh_alu_act |
                DJNZs$t2  == ir2 | (ED_INsREG_6C7 == {ir2[9:6],ir2[2:0]}) & (ir2[5:3] == REG8_B) |
                (ED_LDsREG_6NN7 == {ir2[9:6],ir2[3:0]}) & (ir2[5:4] == DBL_REG_BC);


always @(posedge clk)
begin
    if ( upd_b_alu8 & exec_ir2)        br <= alu8_out;
    if ( up_b_src_pqr & exec_ir2)      br <= src_pqr20;
    if ( up_b_add16   & exec_ir2)      br <= add16[15:8];
    if ( LDsBC_NN  == ir2 & exec_ir2)  br <= nn[15:8];
    if ( POPsBC    == ir2 & exec_ir2)  br <= nn[15:8];
    if ( EXX       == ir2  & exec_ir2) br <= bp;
    if ( LDsB_N    == ir2  & exec_ir2) br <= nn[15:8];
    if (ir2[2:0] == REG8_B & 
             bit_alu_act & exec_ir2)   br <= bit_alu;
    if (ir2[2:0] == REG8_B & 
             sh_alu_act & exec_ir2)    br <= sh_alu;  
    if ( DJNZs$t2  == ir2  & exec_ir2) br <= br + 8'hff; 
                                                        
                                                        
                                                        
    if (exec_decb | exec_decbc |upd_bc_cpi)        br <= decb_alu; 
    if ( (ED_INsREG_6C7 == {ir2[9:6],ir2[2:0]}) & (ir2[5:3] == REG8_B) & exec_ir2 ) br <= nn[15:8];
    if ( (ED_LDsREG_6NN7 == {ir2[9:6],ir2[3:0]}) & (ir2[5:4] == DBL_REG_BC) & exec_ir2 ) br <= nn[15:8];
end



assign up_c_add16 = 
    INCsBC       == ir2 |
    DECsBC       == ir2 ;
assign upd_c_alu8 =
    INCsC        == ir2 |
    DECsC        == ir2 ;
assign up_c_src_pqr =
    LDsC_B       == ir2 |
    LDsC_C       == ir2 |
    LDsC_D       == ir2 |
    LDsC_E       == ir2 |
    LDsC_H       == ir2 |
    LDsC_L       == ir2 |
    LDsC_6HL7    == ir2 |
    LDsC_A       == ir2 ;









assign upd_cr = upd_c_alu8 | up_c_src_pqr | up_c_add16 | LDsBC_NN  == ir2 | 
                POPsBC    == ir2 | EXX       == ir2 | LDsC_N    == ir2    | 
                ir2[2:0] == REG8_C & bit_alu_act | ir2[2:0] == REG8_C & sh_alu_act |
                (ED_INsREG_6C7 == {ir2[9:6],ir2[2:0]}) & (ir2[5:3] == REG8_C)      |
                (ED_LDsREG_6NN7 == {ir2[9:6],ir2[3:0]}) & (ir2[5:4] == DBL_REG_BC);



always @(posedge clk)
begin
    if ( upd_c_alu8 & exec_ir2)        cr <= alu8_out;
    if ( up_c_src_pqr & exec_ir2)      cr <= src_pqr20;
    if ( up_c_add16   & exec_ir2)      cr <= add16[7:0];
    if ( LDsBC_NN  == ir2 & exec_ir2)  cr <= nn[7:0];
    if ( POPsBC    == ir2 & exec_ir2)  cr <= nn[7:0];
    if ( EXX       == ir2  & exec_ir2) cr <= cp;
    if ( LDsC_N    == ir2  & exec_ir2) cr <= nn[15:8];
    if (ir2[2:0] == REG8_C & 
             bit_alu_act & exec_ir2)   cr <= bit_alu;
    if (ir2[2:0] == REG8_C & 
             sh_alu_act & exec_ir2)    cr <= sh_alu;  
    if ( exec_decbc |upd_bc_cpi)       cr <= decc_alu;
    if ((ED_INsREG_6C7 == {ir2[9:6],ir2[2:0]}) & (ir2[5:3] == REG8_C) & exec_ir2)    
                                           cr <= nn[15:8];    
    if ( (ED_LDsREG_6NN7 == {ir2[9:6],ir2[3:0]}) & (ir2[5:4] == DBL_REG_BC) & exec_ir2 ) cr <= nn[7:0];

end



assign up_d_add16 =
    INCsDE       == ir2 |  
    DECsDE       == ir2  ; 

assign upd_d_alu8 =
    INCsD        == ir2 |  
    DECsD        == ir2  ; 
assign up_d_src_pqr =
    LDsD_B       == ir2 |      
    LDsD_C       == ir2 |      
    LDsD_D       == ir2 |      
    LDsD_E       == ir2 |      
    LDsD_H       == ir2 |      
    LDsD_L       == ir2 |      
    LDsD_6HL7    == ir2 |      
    LDsD_A       == ir2 ;      
             










assign upd_dr = upd_d_alu8 | up_d_src_pqr | up_d_add16 | LDsDE_NN  == ir2 | 
                POPsDE    == ir2 | EXX       == ir2 | EXsDE_HL == ir2 | LDsD_N    == ir2 | 
                ir2[2:0] == REG8_D & bit_alu_act | ir2[2:0] == REG8_D & sh_alu_act |
                 (ED_INsREG_6C7 == {ir2[9:6],ir2[2:0]}) & (ir2[5:3] == REG8_D)     |
                 (ED_LDsREG_6NN7 == {ir2[9:6],ir2[3:0]}) & (ir2[5:4] == DBL_REG_DE);





wire ed_ld_dereg = (ED_LDsREG_6NN7 == {ir2[9:6],ir2[3:0]}) & (ir2[5:4] == DBL_REG_DE);
always @(posedge clk)
begin
    if ( upd_d_alu8 & exec_ir2)        dr <= alu8_out;
    if ( up_d_src_pqr & exec_ir2)      dr <= src_pqr20;
    if ( up_d_add16   & exec_ir2)      dr <= add16[15:8];
    if ( LDsDE_NN  == ir2 & exec_ir2)  dr <= nn[15:8];
    if ( POPsDE    == ir2 & exec_ir2)  dr <= nn[15:8];
    if ( EXX       == ir2  & exec_ir2) dr <= dp;
    if ( EXsDE_HL  == ir2  & exec_ir2) dr <= hr;
    if ( LDsD_N    == ir2  & exec_ir2) dr <= nn[15:8];
    if (ir2[2:0] == REG8_D & 
             bit_alu_act & exec_ir2)   dr <= bit_alu;
    if (ir2[2:0] == REG8_D & 
             sh_alu_act & exec_ir2)    dr <= sh_alu;  
    if ((ED_INsREG_6C7 == {ir2[9:6],ir2[2:0]}) 
         & (ir2[5:3] == REG8_D) & exec_ir2)             
                                        dr <= nn[15:8];
    if ( ed_ld_dereg & exec_ir2 ) 
                                        dr <= nn[15:8];
    if (blk_mv_upd_de)                  dr <= adr_alu[15:8];
end


assign up_e_add16 =
    INCsDE       == ir2 |
    DECsDE       == ir2 ;
assign upd_e_alu8 =
    INCsE        == ir2 |
    DECsE        == ir2 ;
assign up_e_src_pqr =
    LDsE_B       == ir2 |
    LDsE_C       == ir2 |
    LDsE_D       == ir2 |
    LDsE_E       == ir2 |
    LDsE_H       == ir2 |
    LDsE_L       == ir2 |
    LDsE_6HL7    == ir2 |
    LDsE_A       == ir2 ;











assign upd_er = upd_e_alu8 | up_e_src_pqr | up_e_add16 | LDsDE_NN  == ir2 | 
                POPsDE    == ir2 | EXX       == ir2 | EXsDE_HL == ir2 | LDsE_N    == ir2 | 
                ir2[2:0] == REG8_E & bit_alu_act | ir2[2:0] == REG8_E & sh_alu_act |
                 (ED_INsREG_6C7 == {ir2[9:6],ir2[2:0]}) & (ir2[5:3] == REG8_E)     |
                 (ED_LDsREG_6NN7 == {ir2[9:6],ir2[3:0]}) & (ir2[5:4] == DBL_REG_DE);

always @(posedge clk)
begin
    if ( upd_e_alu8 & exec_ir2)        er <= alu8_out;
    if ( up_e_src_pqr & exec_ir2)      er <= src_pqr20;
    if ( up_e_add16   & exec_ir2)      er <= add16[7:0];
    if ( LDsDE_NN  == ir2 & exec_ir2)  er <= nn[7:0];
    if ( POPsDE    == ir2 & exec_ir2)  er <= nn[7:0];
    if ( EXX       == ir2  & exec_ir2) er <= ep;
    if ( EXsDE_HL  == ir2  & exec_ir2) er <= hr;
    if ( LDsE_N    == ir2  & exec_ir2) er <= nn[15:8];
    if (ir2[2:0] == REG8_E & 
             bit_alu_act & exec_ir2)   er <= bit_alu;
    if (ir2[2:0] == REG8_E & 
             sh_alu_act & exec_ir2)    er <= sh_alu;
    if ((ED_INsREG_6C7 == {ir2[9:6],ir2[2:0]}) & (ir2[5:3] == REG8_E) & exec_ir2)              
                                           er <= nn[15:8];             
    if ( ed_ld_dereg & exec_ir2 ) 
                                        er <= nn[7:0];
    if (blk_mv_upd_de)                  er <= adr_alu[7:0];
     
end



assign up_h_add16 =
    ADDsHL_BC    == ir2 | 
    ADDsHL_DE    == ir2 | 
    ADDsHL_HL    == ir2 | 
    ADDsHL_SP    == ir2 | 
    ED_SBCsHL_REG  == {ir2[9:6],ir2[3:0]}  |  
    ED_ADCsHL_REG  == {ir2[9:6],ir2[3:0]}  |  

    INCsHL       == ir2 | 
    DECsHL       == ir2 ; 
assign upd_h_alu8 =
    INCsH        == ir2 | 
    DECsH        == ir2 ; 
assign upd_h_src_pqr =
    LDsH_B       == ir2 | 
    LDsH_C       == ir2 | 
    LDsH_D       == ir2 | 
    LDsH_E       == ir2 | 
    LDsH_H       == ir2 | 
    LDsH_L       == ir2 | 
    LDsH_6HL7    == ir2 | 
    LDsH_A       == ir2 ; 

















assign upd_hr = upd_h_alu8 | upd_h_src_pqr | up_h_add16 | LDsHL_NN  == ir2 | LDsHL_6NN7== ir2 |
                POPsHL    == ir2 | EXX       == ir2 | EXsDE_HL == ir2 | LDsH_N    == ir2 | 
                ir2[2:0] == REG8_H & bit_alu_act | ir2[2:0] == REG8_H & sh_alu_act |
                 (ED_INsREG_6C7 == {ir2[9:6],ir2[2:0]}) & (ir2[5:3] == REG8_H)     |
                 (ED_LDsREG_6NN7 == {ir2[9:6],ir2[3:0]}) & (ir2[5:4] == DBL_REG_HL);




wire exec_hlir2 = exec_ir2 & !(ir2dd | ir2fd);

always @(posedge clk)
begin
    if ( upd_h_alu8 & exec_hlir2)        hr <= alu8_out;
    if ( upd_h_src_pqr & exec_hlir2)      hr <= src_pqr20;
    if ( up_h_add16   & exec_hlir2)      hr <= add16[15:8];
    if ( LDsHL_NN  == ir2 & exec_hlir2)  hr <= nn[15:8];
    if ( LDsHL_6NN7== ir2 & exec_hlir2)  hr <= nn[15:8];
    if ( POPsHL    == ir2 & exec_hlir2)  hr <= nn[15:8];
    if ( EXs6SP7_HL== ir2 & exec_hlir2)  hr <= nn[15:8];
    if ( EXX       == ir2  & exec_ir2)   hr <= hp;
    if ( EXsDE_HL  == ir2  & exec_ir2)   hr <= dr;
    if ( LDsH_N    == ir2  & exec_hlir2) hr <= nn[15:8];
    if (ir2[2:0] == REG8_H & 
             bit_alu_act & exec_hlir2)   hr <= bit_alu;
    if (ir2[2:0] == REG8_H & 
             sh_alu_act & exec_hlir2)    hr <= sh_alu;  
    if ((ED_INsREG_6C7 == {ir2[9:6],ir2[2:0]}) & (ir2[5:3] == REG8_H) & exec_ir2)              
                                           hr <= nn[15:8];
    if ( (ED_LDsREG_6NN7 == {ir2[9:6],ir2[3:0]}) & (ir2[5:4] == DBL_REG_HL) & exec_ir2 ) 
                                         hr <= nn[15:8];
    if (blk_mv_upd_hl)                   hr <= adr_alu[15:8];

end


assign up_l_add16 =
    ADDsHL_BC    == ir2 |
    ADDsHL_DE    == ir2 |
    ADDsHL_HL    == ir2 |
    ADDsHL_SP    == ir2 |
    ED_SBCsHL_REG  == {ir2[9:6],ir2[3:0]}  |  
    ED_ADCsHL_REG  == {ir2[9:6],ir2[3:0]}  |  
    INCsHL       == ir2 |
    DECsHL       == ir2 ;
assign upd_l_alu8 =
    INCsL        == ir2 |
    DECsL        == ir2 ;
assign upd_l_src_pqr =
    LDsL_B       == ir2 |
    LDsL_C       == ir2 |
    LDsL_D       == ir2 |
    LDsL_E       == ir2 |
    LDsL_H       == ir2 |
    LDsL_L       == ir2 |
    LDsL_6HL7    == ir2 |
    LDsL_A       == ir2 ;












assign upd_lr = upd_l_alu8 | upd_l_src_pqr | up_l_add16 | LDsHL_NN  == ir2 | LDsHL_6NN7== ir2 |
                POPsHL    == ir2 | EXX       == ir2 | EXsDE_HL == ir2 | LDsL_N    == ir2 | 
                ir2[2:0] == REG8_L & bit_alu_act | ir2[2:0] == REG8_L & sh_alu_act |
                 (ED_INsREG_6C7 == {ir2[9:6],ir2[2:0]}) & (ir2[5:3] == REG8_L)     |
                 (ED_LDsREG_6NN7 == {ir2[9:6],ir2[3:0]}) & (ir2[5:4] == DBL_REG_HL);



always @(posedge clk)
begin
    if ( upd_l_alu8 & exec_hlir2)        lr <= alu8_out;
    if ( upd_l_src_pqr & exec_hlir2)      lr <= src_pqr20;
    if ( up_l_add16   & exec_hlir2)      lr <= add16[7:0];
    if ( LDsHL_NN  == ir2 & exec_hlir2)  lr <= nn[7:0];
    if ( LDsHL_6NN7== ir2 & exec_hlir2)  lr <= nn[7:0];
    if ( POPsHL    == ir2 & exec_hlir2)  lr <= nn[7:0];
    if ( EXs6SP7_HL== ir2 & exec_hlir2)  lr <= nn[7:0];
    if ( EXX       == ir2  & exec_ir2)   lr <= lp;
    if ( EXsDE_HL  == ir2  & exec_ir2)   lr <= er;
    if ( LDsL_N    == ir2  & exec_hlir2) lr <= nn[15:8];
    if (ir2[2:0] == REG8_L & 
             bit_alu_act & exec_hlir2)   lr <= bit_alu;
    if (ir2[2:0] == REG8_L & 
             sh_alu_act & exec_hlir2)    lr <= sh_alu;
    if ((ED_INsREG_6C7 == {ir2[9:6],ir2[2:0]}) & (ir2[5:3] == REG8_L) & exec_ir2)              
                                         lr <= nn[15:8];             
    if ( (ED_LDsREG_6NN7 == {ir2[9:6],ir2[3:0]}) & (ir2[5:4] == DBL_REG_HL) & exec_ir2 ) 
                                         lr <= nn[7:0];
    if (blk_mv_upd_hl)                   lr <= adr_alu[7:0];
     
end

wire exec_ixir2 = exec_ir2 & ir2dd;
always @(posedge clk)
begin
    if ( upd_l_alu8 & exec_ixir2)        ixr[7:0] <= alu8_out;
    if ( upd_l_src_pqr & exec_ixir2)      ixr[7:0] <= src_pqr20;
    if ( up_l_add16   & exec_ixir2)      ixr[7:0] <= add16[7:0];
    if ( LDsHL_NN  == ir2 & exec_ixir2)  ixr[7:0] <= nn[7:0];
    if ( LDsHL_6NN7== ir2 & exec_ixir2)  ixr[7:0] <= nn[7:0];
    if ( POPsHL    == ir2 & exec_ixir2)  ixr[7:0] <= nn[7:0];
    if ( EXs6SP7_HL== ir2 & exec_ixir2)  ixr[7:0] <= nn[7:0];

    if ( LDsL_N    == ir2  & exec_ixir2) ixr[7:0] <= nn[15:8];
    if (ir2[2:0] == REG8_L & 
             bit_alu_act & exec_ixir2)   ixr[7:0] <= bit_alu;
    if (ir2[2:0] == REG8_L & 
             sh_alu_act & exec_ixir2)    ixr[7:0] <= sh_alu;  
             
end

always @(posedge clk)
begin
    if ( upd_h_alu8 & exec_ixir2)        ixr[15:8] <= alu8_out;
    if ( upd_h_src_pqr & exec_ixir2)      ixr[15:8] <= src_pqr20;
    if ( up_h_add16   & exec_ixir2)      ixr[15:8] <= add16[15:8];
    if ( LDsHL_NN  == ir2 & exec_ixir2)  ixr[15:8] <= nn[15:8];
    if ( LDsHL_6NN7== ir2 & exec_ixir2)  ixr[15:8] <= nn[15:8];
    if ( POPsHL    == ir2 & exec_ixir2)  ixr[15:8] <= nn[15:8];
    if ( EXs6SP7_HL== ir2 & exec_ixir2)  ixr[15:8] <= nn[15:8];

    if ( LDsH_N    == ir2  & exec_ixir2) ixr[15:8] <= nn[15:8];
    if (ir2[2:0] == REG8_H & 
             bit_alu_act & exec_ixir2)   ixr[15:8] <= bit_alu;
    if (ir2[2:0] == REG8_H & 
             sh_alu_act & exec_ixir2)    ixr[15:8] <= sh_alu;  
             
end


wire exec_iyir2 = exec_ir2 & ir2fd;
always @(posedge clk)
begin
    if ( upd_l_alu8 & exec_iyir2)        iyr[7:0] <= alu8_out;
    if ( upd_l_src_pqr & exec_iyir2)      iyr[7:0] <= src_pqr20;
    if ( up_l_add16   & exec_iyir2)      iyr[7:0] <= add16[7:0];
    if ( LDsHL_NN  == ir2 & exec_iyir2)  iyr[7:0] <= nn[7:0];
    if ( LDsHL_6NN7== ir2 & exec_iyir2)  iyr[7:0] <= nn[7:0];
    if ( POPsHL    == ir2 & exec_iyir2)  iyr[7:0] <= nn[7:0];
    if ( EXs6SP7_HL== ir2 & exec_iyir2)  iyr[7:0] <= nn[7:0];

    if ( LDsL_N    == ir2  & exec_iyir2) iyr[7:0] <= nn[15:8];
    if (ir2[2:0] == REG8_L & 
             bit_alu_act & exec_iyir2)   iyr[7:0] <= bit_alu;
    if (ir2[2:0] == REG8_L & 
             sh_alu_act & exec_iyir2)    iyr[7:0] <= sh_alu;  
             
end

always @(posedge clk)
begin
    if ( upd_h_alu8 & exec_iyir2)        iyr[15:8] <= alu8_out;
    if ( upd_h_src_pqr & exec_iyir2)      iyr[15:8] <= src_pqr20;
    if ( up_h_add16   & exec_iyir2)      iyr[15:8] <= add16[15:8];
    if ( LDsHL_NN  == ir2 & exec_iyir2)  iyr[15:8] <= nn[15:8];
    if ( LDsHL_6NN7== ir2 & exec_iyir2)  iyr[15:8] <= nn[15:8];
    if ( POPsHL    == ir2 & exec_iyir2)  iyr[15:8] <= nn[15:8];
    if ( EXs6SP7_HL== ir2 & exec_iyir2)  iyr[15:8] <= nn[15:8];

    if ( LDsH_N    == ir2  & exec_iyir2) iyr[15:8] <= nn[15:8];
    if (ir2[2:0] == REG8_H & 
             bit_alu_act & exec_iyir2)   iyr[15:8] <= bit_alu;
    if (ir2[2:0] == REG8_H & 
             sh_alu_act & exec_iyir2)    iyr[15:8] <= sh_alu;  
             
end




always @(posedge clk)
begin
    if (ir2 == EXsAF_AFp & exec_ir2) 
    begin
        ap <= ar; 
        fp <= fr; 
    end
    if (ir2 == EXX & exec_ir2)       
    begin
        ap <= ar;
        fp <= fr;
        bp <= br;
        cp <= cr;
        dp <= dr;
        ep <= er;
        hp <= hr;
        lp <= lr;
    end
end
























assign upd_fr_alu8 = 
    ADCsA_A  == ir2 |   ANDsA    == ir2 |  ORsA     == ir2 | SUBsA    == ir2 | DECsA       == ir2 |
    ADCsA_B  == ir2 |   ANDsB    == ir2 |  ORsB     == ir2 | SUBsB    == ir2 | DECsB       == ir2 |
    ADCsA_C  == ir2 |   ANDsC    == ir2 |  ORsC     == ir2 | SUBsC    == ir2 | DECsC       == ir2 |
    ADCsA_D  == ir2 |   ANDsD    == ir2 |  ORsD     == ir2 | SUBsD    == ir2 | DECsD       == ir2 |
    ADCsA_E  == ir2 |   ANDsE    == ir2 |  ORsE     == ir2 | SUBsE    == ir2 | DECsE       == ir2 |
    ADCsA_H  == ir2 |   ANDsH    == ir2 |  ORsH     == ir2 | SUBsH    == ir2 | DECsH       == ir2 |
    ADCsA_L  == ir2 |   ANDsL    == ir2 |  ORsL     == ir2 | SUBsL    == ir2 | DECsL       == ir2 |
    ADCsA_6HL7==ir2 |   ANDs6HL7  ==ir2 |  ORs6HL7  ==ir2  | SUBs6HL7  ==ir2 | INCsA       == ir2 |
    ADDsA_A  == ir2 |   CPsA     == ir2 |  SBCsA    == ir2 | XORsA    == ir2 | INCsB       == ir2 |
    ADDsA_B  == ir2 |   CPsB     == ir2 |  SBCs6HL7  ==ir2 | XORsB    == ir2 | INCsC       == ir2 |
    ADDsA_C  == ir2 |   CPsC     == ir2 |  SBCsB    == ir2 | XORsC    == ir2 | INCsD       == ir2 |
    ADDsA_D  == ir2 |   CPsD     == ir2 |  SBCsC    == ir2 | XORsD    == ir2 | INCsE       == ir2 |
    ADDsA_E  == ir2 |   CPsE     == ir2 |  SBCsD    == ir2 | XORsE    == ir2 | INCsH       == ir2 |
    ADDsA_H  == ir2 |   CPsH     == ir2 |  SBCsE    == ir2 | XORsH    == ir2 | INCsL       == ir2 |
    ADDsA_L  == ir2 |   CPsL     == ir2 |  SBCsH    == ir2 | XORsL    == ir2 | INCs6HL7    == ir2 |
    ADDsA_6HL7== ir2|   CPs6HL7  == ir2 |  SBCsL    == ir2 | XORs6HL7 == ir2 | DECs6HL7    == ir2 |
    ADDsA_N   == ir2|   SUBsN    == ir2 | ANDsN     == ir2 |  ORsN    == ir2 |
    ADCsA_N   ==ir2 |   SBCsA_N  ==ir2  | XORsN     ==ir2  |  CPsN    == ir2 |
    ED_NEG   ==  {ir2[9:6],ir2[2:0]} ; 

                                       


assign upd_fr_add16 = 
    ADDsHL_BC    == ir2 |      
    ADDsHL_DE    == ir2 |      
    ADDsHL_HL    == ir2 |      
    ADDsHL_SP    == ir2 ;      






assign upd_fr_edadd16 = 
    ED_SBCsHL_REG  == {ir2[9:6],ir2[3:0]} | 
    ED_ADCsHL_REG  == {ir2[9:6],ir2[3:0]} ; 

wire borrow = ED_SBCsHL_REG  == {ir2[9:6],ir2[3:0]};



assign upd_fr_sh =
    RLA          == ir2 |
    RLCA         == ir2 |
    RRA          == ir2 |
    RRCA         == ir2 ;

assign upd_fr_cbsh = 
    CB_RLC   == ir2[9:3] |  
    CB_RRC   == ir2[9:3] |  
    CB_RL    == ir2[9:3] |  
    CB_RR    == ir2[9:3] |  
    CB_SLA   == ir2[9:3] |  
    CB_SRA   == ir2[9:3] |  
    CB_SLL   == ir2[9:3] |  
    CB_SRL   == ir2[9:3] ;  


















assign ed_blk_cp =
    ED_CPI       == ir2 | 
    ED_CPD       == ir2 | 
    ED_CPIR      == ir2 | 
    ED_CPDR      == ir2  ;


























assign { sf, zf, f5f, hf, f3f, pvf, nf,  cf} = fr;






assign upd_fr =  exec_ir2 & ( ( upd_fr_alu8 )                       |
                              ( upd_fr_add16)                       |
                              ( upd_fr_edadd16)                     |
                              ( upd_fr_sh )                         |
                              ( upd_fr_cbsh )                       |
                              (CB_BIT == ir2[9:6])                  |
                              ( ed_blk_cp )                         |
                              (ED_INsREG_6C7 == {ir2[9:6],ir2[2:0]})  |
                              (CCF == ir2 )                         |
                              (CPL == ir2 )                         |
                              (DAA == ir2 )                         |
                              (SCF == ir2 )                         |
                              (ED_RRD == ir2)                       |
                              (ED_RLD == ir2)                       |
                              (ED_LDsA_I == ir2)                    | 
                              (ir2 == EXsAF_AFp )                   |  
                              (ir2 == EXX )                         );



wire iff2 = 1'b0; 
wire upd_fr_ed_in =  ED_INsREG_6C7 == {ir2[9:6],ir2[2:0]} ;
wire bc_eq1 = {br,cr} == 16'h1;
always @(posedge clk)
begin
    if (exec_ir2)
    begin
        if ( upd_fr_alu8 )      fr <= alu8_fr;    
        if ( upd_fr_add16)      fr <= {sf, zf, add16[13], c_16out11, add16[11], pvf, 1'b0, c_16out15};
        if ( upd_fr_edadd16)    fr <= {add16[15],   ~|add16, add16[13], c_16out11, 
                                       add16[11], add16_ofl,   ~ir2[3], borrow ^ c_16out15};
        if ( upd_fr_sh )        fr <= {sf, zf, sh_alu[5], 1'b0, sh_alu[3], pvf, 1'b0, sh_cry};
        if ( upd_fr_cbsh )      fr <= {sh_alu[7], ~|sh_alu, sh_alu[5], 1'b0, 
                                       sh_alu[3], ~^sh_alu,      1'b0, sh_cry};
        if (CB_BIT == ir2[9:6]) fr <={bit_alu[7], ~|bit_alu, bit_alu[5], 1'b1, 
                                      bit_alu[3], ~|bit_alu, 1'b0      , cf  };
        if ( ed_blk_cp )        fr <= {alu8_out[7], ~|alu8_out, alu8_out[5], alu8_hcry,
                                   alu8_out[3], ~bc_eq1,       1'b1,  cf };    
        if (upd_fr_ed_in)
                                fr <= {nn[15], ~|nn[15:8], nn[13], 1'b0, nn[11], ~^nn[15:8], 1'b0, cf};
        if (CCF == ir2 )        fr <= {sf, zf, f5f, cf, f3f, pvf, nf, ~cf};
        if (CPL == ir2 )        fr <= {sf, zf, ar[5], 1'b1, ar[3], pvf, 1'b1, cf};
        if (DAA == ir2 )        fr <= {daa_alu[7], ~|daa_alu, daa_alu[5], 1'b0, 
                                    daa_alu[3], ~^daa_alu,         nf, daa_cry };
        if (SCF == ir2 )        fr <= { sf, zf, ar[5], 1'b0, ar[3], pvf, 1'b0, 1'b1 }; 
        if (ED_RRD == ir2)      fr <= {     ar[7], ~|{ar[7:4],nn[11:8]}, ar[5], 1'b0, 
                                     ar[3],  ~^{ar[7:4],nn[11:8]}, 1'b0 , cf    };
        if (ED_RLD == ir2)      fr <= {     ar[7], ~|{ar[7:4],nn[15:12]}, ar[5], 1'b0, 
                                     ar[3],  ~^{ar[7:4],nn[15:12]}, 1'b0 , cf    };
        if (ED_LDsA_I == ir2)   fr <= { intr[7], ~|intr, intr[5], 1'b0, intr[3], iff2, 1'b0, cf }; 
        if (ir2 == EXsAF_AFp)   fr <= fp;
        if (ir2 == EXX  )       fr <= fp;
        if (ir2 == POPsAF)      fr <= nn[7:0];

    
    end
    
    if (exec_decb )          fr <=  {decb_alu[7], ~|decb_alu, decb_alu[5], hf,
                                     decb_alu[3],        pvf,        1'b0, cf };
    if (exec_decbc )         fr[5:1] <= { decb_alu[5], 1'b0, decb_alu[3],
                                           ((|decb_alu) | (|decc_alu)) , 1'b0 };
end    
    
    
 
 
always @(posedge clk or posedge rst)
    if (rst) intr <= 8'h0;
    else if (( ED_LDsI_A == ir2) & exec_ir2) intr <= ar;

endmodule


`line 1 "F:\verilog\wb_z80\new_wb_z80\wb_z80\rtl\z80_memstate2.v"







































































































































































module z80_memstate2(wb_adr_o, wb_we_o, wb_cyc_o, wb_stb_o, wb_tga_o, wb_dat_o,
                exec_ir2, 
                exec_decbc, exec_decb,
                ir1, ir2, ir1dd, ir1fd, ir2dd, ir2fd, nn, sp,
                
                upd_ar, upd_br, upd_cr, upd_dr, upd_er, upd_hr, upd_lr,upd_fr,
                beq0, ceq0,
                ar, fr, br, cr, dr, er, hr, lr, intr, 
                ixr, iyr, 
                wb_dat_i, wb_ack_i, wb_clk_i, 
                int_req_i,
                add16,
                alu8_out,
                adr_alu,   
                blk_mv_upd_hl,
                blk_mv_upd_de,
                sh_alu,
                bit_alu,
                rst_i


);






output [15:0]  wb_adr_o; 
output         wb_we_o; 
output         wb_cyc_o; 
output         wb_stb_o; 

output [1:0]   wb_tga_o;
output [7:0]   wb_dat_o;   
output [15:0]  adr_alu;    
                           

output         blk_mv_upd_hl;
output         blk_mv_upd_de;
output         exec_ir2;
output [9:0]   ir1, ir2;
output         ir1dd, ir2dd;
output          ir1fd, ir2fd;
output [15:0]   nn;
output [15:0]   sp;
output          exec_decbc, exec_decb; 





input           upd_ar, upd_br, upd_cr, upd_dr, upd_er, upd_hr, upd_lr,upd_fr;

input           beq0, ceq0;
input [7:0]     ar, fr, br, cr, dr, er, hr, lr, intr;
input [15:0]    ixr, iyr;
input [7:0]     wb_dat_i;
input           wb_ack_i, wb_clk_i, rst_i;
input           int_req_i;     
input [15:0]    add16;         
input [7:0]     alu8_out;
input [7:0]     sh_alu;        
input [7:0]     bit_alu;

`line 1 "F:\verilog\wb_z80\new_wb_z80\wb_z80\rtl\opcodes.v"































































































parameter  NOP          = 10'h00,
           LDsBC_NN     = 10'h01,
           LDs6BC7_A    = 10'h02,
           INCsBC       = 10'h03,
           INCsB        = 10'h04,
           DECsB        = 10'h05,
           LDsB_N       = 10'h06,
           RLCA         = 10'h07,
           EXsAF_AFp    = 10'h08,
           ADDsHL_BC    = 10'h09,
           LDsA_6BC7    = 10'h0A,
           DECsBC       = 10'h0B,
           INCsC        = 10'h0C,
           DECsC        = 10'h0D,
           LDsC_N       = 10'h0E,
           RRCA         = 10'h0F,
           DJNZs$t2     = 10'h10,
           LDsDE_NN     = 10'h11,
           LDs6DE7_A    = 10'h12,
           INCsDE       = 10'h13,
           INCsD        = 10'h14,
           DECsD        = 10'h15,
           LDsD_N       = 10'h16,
           RLA          = 10'h17,
           JRs$t2       = 10'h18,
           ADDsHL_DE    = 10'h19,
           LDsA_6DE7    = 10'h1A,
           DECsDE       = 10'h1B,
           INCsE        = 10'h1C,
           DECsE        = 10'h1D,
           LDsE_N       = 10'h1E,
           RRA          = 10'h1F,
           JRsNZ_$t2    = 10'h20,
           LDsHL_NN     = 10'h21,
           LDs6NN7_HL   = 10'h22,
           INCsHL       = 10'h23,
           INCsH        = 10'h24,
           DECsH        = 10'h25,
           LDsH_N       = 10'h26,
           DAA          = 10'h27,
           JRsZ_$t2     = 10'h28,
           ADDsHL_HL    = 10'h29,
           LDsHL_6NN7   = 10'h2A,
           DECsHL       = 10'h2B,
           INCsL        = 10'h2C,
           DECsL        = 10'h2D,
           LDsL_N       = 10'h2E,
           CPL          = 10'h2F,
           JRsNC_$t2    = 10'h30,
           LDsSP_NN     = 10'h31,
           LDs6NN7_A    = 10'h32,
           INCsSP       = 10'h33,
           INCs6HL7     = 10'h34,
           DECs6HL7     = 10'h35,
           LDs6HL7_N    = 10'h36,
           SCF          = 10'h37,
           JRsC_$t2     = 10'h38,
           ADDsHL_SP    = 10'h39,
           LDsA_6NN7    = 10'h3A,
           DECsSP       = 10'h3B,
           INCsA        = 10'h3C,
           DECsA        = 10'h3D,
           LDsA_N       = 10'h3E,
           CCF          = 10'h3F,
           LDsB_B       = 10'h40,
           LDsB_C       = 10'h41,
           LDsB_D       = 10'h42,
           LDsB_E       = 10'h43,
           LDsB_H       = 10'h44,
           LDsB_L       = 10'h45,
           LDsB_6HL7    = 10'h46,
           LDsB_A       = 10'h47,
           LDsC_B       = 10'h48,
           LDsC_C       = 10'h49,
           LDsC_D       = 10'h4A,
           LDsC_E       = 10'h4B,
           LDsC_H       = 10'h4C,
           LDsC_L       = 10'h4D,
           LDsC_6HL7    = 10'h4E,
           LDsC_A       = 10'h4F,
           LDsD_B       = 10'h50,
           LDsD_C       = 10'h51,
           LDsD_D       = 10'h52,
           LDsD_E       = 10'h53,
           LDsD_H       = 10'h54,
           LDsD_L       = 10'h55,
           LDsD_6HL7    = 10'h56,
           LDsD_A       = 10'h57,
           LDsE_B       = 10'h58,
           LDsE_C       = 10'h59,
           LDsE_D       = 10'h5A,
           LDsE_E       = 10'h5B,
           LDsE_H       = 10'h5C,
           LDsE_L       = 10'h5D,
           LDsE_6HL7    = 10'h5E,
           LDsE_A       = 10'h5F,
           LDsH_B       = 10'h60,
           LDsH_C       = 10'h61,
           LDsH_D       = 10'h62,
           LDsH_E       = 10'h63,
           LDsH_H       = 10'h64,
           LDsH_L       = 10'h65,
           LDsH_6HL7    = 10'h66,
           LDsH_A       = 10'h67,
           LDsL_B       = 10'h68,
           LDsL_C       = 10'h69,
           LDsL_D       = 10'h6A,
           LDsL_E       = 10'h6B,
           LDsL_H       = 10'h6C,
           LDsL_L       = 10'h6D,
           LDsL_6HL7    = 10'h6E,
           LDsL_A       = 10'h6F,
           LDs6HL7_B    = 10'h70,
           LDs6HL7_C    = 10'h71,
           LDs6HL7_D    = 10'h72,
           LDs6HL7_E    = 10'h73,
           LDs6HL7_H    = 10'h74,
           LDs6HL7_L    = 10'h75,
           HALT         = 10'h76,
           LDs6HL7_A    = 10'h77,
           LDsA_B       = 10'h78,
           LDsA_C       = 10'h79,
           LDsA_D       = 10'h7A,
           LDsA_E       = 10'h7B,
           LDsA_H       = 10'h7C,
           LDsA_L       = 10'h7D,
           LDsA_6HL7    = 10'h7E,
           LDsA_A       = 10'h7F,
           ADDsA_B      = 10'h80,
           ADDsA_C      = 10'h81,
           ADDsA_D      = 10'h82,
           ADDsA_E      = 10'h83,
           ADDsA_H      = 10'h84,
           ADDsA_L      = 10'h85,
           ADDsA_6HL7   = 10'h86,
           ADDsA_A      = 10'h87,
           ADCsA_B      = 10'h88,
           ADCsA_C      = 10'h89,
           ADCsA_D      = 10'h8A,
           ADCsA_E      = 10'h8B,
           ADCsA_H      = 10'h8C,
           ADCsA_L      = 10'h8D,
           ADCsA_6HL7   = 10'h8E,
           ADCsA_A      = 10'h8F,
           SUBsB        = 10'h90,
           SUBsC        = 10'h91,
           SUBsD        = 10'h92,
           SUBsE        = 10'h93,
           SUBsH        = 10'h94,
           SUBsL        = 10'h95,
           SUBs6HL7     = 10'h96,
           SUBsA        = 10'h97,
           SBCsB        = 10'h98,
           SBCsC        = 10'h99,
           SBCsD        = 10'h9A,
           SBCsE        = 10'h9B,
           SBCsH        = 10'h9C,
           SBCsL        = 10'h9D,
           SBCs6HL7     = 10'h9E,
           SBCsA        = 10'h9F,
           ANDsB        = 10'hA0,
           ANDsC        = 10'hA1,
           ANDsD        = 10'hA2,
           ANDsE        = 10'hA3,
           ANDsH        = 10'hA4,
           ANDsL        = 10'hA5,
           ANDs6HL7     = 10'hA6,
           ANDsA        = 10'hA7,
           XORsB        = 10'hA8,
           XORsC        = 10'hA9,
           XORsD        = 10'hAA,
           XORsE        = 10'hAB,
           XORsH        = 10'hAC,
           XORsL        = 10'hAD,
           XORs6HL7     = 10'hAE,
           XORsA        = 10'hAF,
           ORsB         = 10'hB0,
           ORsC         = 10'hB1,
           ORsD         = 10'hB2,
           ORsE         = 10'hB3,
           ORsH         = 10'hB4,
           ORsL         = 10'hB5,
           ORs6HL7      = 10'hB6,
           ORsA         = 10'hB7,
           CPsB         = 10'hB8,
           CPsC         = 10'hB9,
           CPsD         = 10'hBA,
           CPsE         = 10'hBB,
           CPsH         = 10'hBC,
           CPsL         = 10'hBD,
           CPs6HL7      = 10'hBE,
           CPsA         = 10'hBF,
           RETsNZ       = 10'hC0,
           POPsBC       = 10'hC1,
           JPsNZ        = 10'hC2,
           JP           = 10'hC3,
           CALLsNZ_NN   = 10'hC4,
           PUSHsBC      = 10'hC5,
           ADDsA_N      = 10'hC6,
           RSTs0        = 10'hC7,
           RETsZ        = 10'hC8,
           RET          = 10'hC9,
           JPsZ         = 10'hCA,
           CALLsZ_NN    = 10'hCC,
           CBgrp        = 10'hCB,
           CALLsNN      = 10'hCD,
           ADCsA_N      = 10'hCE,
           RSTs8H       = 10'hCF,
           RETsNC       = 10'hD0,
           POPsDE       = 10'hD1,
           JPsNC        = 10'hD2,
           OUTs6N7_A    = 10'hD3,
           CALLsNC_NN   = 10'hD4,
           PUSHsDE      = 10'hD5,
           SUBsN        = 10'hD6,
           RSTs10H      = 10'hD7,
           RETsC        = 10'hD8,
           EXX          = 10'hD9,
           JPsC         = 10'hDA,
           INsA_6N7     = 10'hDB,
           CALLsC_NN    = 10'hDC,
           DDgrp        = 10'hDD,
           SBCsA_N      = 10'hDE,
           RSTs18H      = 10'hDF,
           RETsPO       = 10'hE0,
           POPsHL       = 10'hE1,
           JPsPO        = 10'hE2,
           EXs6SP7_HL   = 10'hE3,
           CALLsPO_NN   = 10'hE4,
           PUSHsHL      = 10'hE5,
           ANDsN        = 10'hE6,
           RSTs20H      = 10'hE7,
           RETsPE       = 10'hE8,
           JPsHL        = 10'hE9,
           JPsPE        = 10'hEA,
           EXsDE_HL     = 10'hEB,
           CALLsPE_NN   = 10'hEC,
           EDgrp        = 10'hED,
           XORsN        = 10'hEE,
           RSTs28H      = 10'hEF,
           RETsP        = 10'hF0,
           POPsAF       = 10'hF1,
           JPsP         = 10'hF2,
           DI           = 10'hF3,
           CALLsP_NN    = 10'hF4,
           PUSHsAF      = 10'hF5,
           ORsN         = 10'hF6,
           RSTs30H      = 10'hF7,
           RETsM        = 10'hF8,
           LDsSP_HL     = 10'hF9,
           JPsM         = 10'hFA,
           EI           = 10'hFB,
           CALLsM_NN    = 10'hFC,
           FDgrp        = 10'hFD,
           CPsN         = 10'hFE,
           RSTs38H      = 10'hFF,




           CB_RLC   = 7'b01_00_000,  
           CB_RRC   = 7'b01_00_001,  
           CB_RL    = 7'b01_00_010,  
           CB_RR    = 7'b01_00_011,  
           CB_SLA   = 7'b01_00_100,  
           CB_SRA   = 7'b01_00_101,  
           CB_SLL   = 7'b01_00_110,  
           CB_SRL   = 7'b01_00_111,  
           
           CB_BIT   = 4'b01_01,    
           CB_RES   = 4'b01_10,    
           CB_SET   = 4'b01_11,    
           
           CB_MEM   = 3'b110,   
                             
                        





           ED_IMs0      =  10'h246,  
           ED_LDsI_A    =  10'h247,  
           ED_IMs1      =  10'h256,  
           ED_LDsA_I    =  10'h257,  
           ED_IMs2      =  10'h25E,  
           ED_RRD       =  10'h267,  
           ED_RLD       =  10'h26F,  
                                    
           
           ED_LDI       =  10'h2A0,  
           ED_CPI       =  10'h2A1,  
           ED_INI       =  10'h2A2,  
           ED_OUTI      =  10'h2A3,  
           ED_LDD       =  10'h2A8,  
           ED_CPD       =  10'h2A9,  
           ED_IND       =  10'h2AA,  
           ED_OUTD      =  10'h2AB,  
           ED_LDIR      =  10'h2B0,  
           ED_CPIR      =  10'h2B1,  
           ED_INIR      =  10'h2B2,  
           ED_OTIR      =  10'h2B3,  
           ED_LDDR      =  10'h2B8,  
           ED_CPDR      =  10'h2B9,  
           ED_INDR      =  10'h2BA,  
           ED_OTDR      =  10'h2BB,  





























        ED_INsREG_6C7  =    7'b1001___000, 
        ED_OUTs6C7_REG =    7'b1001___001, 
        ED_SBCsHL_REG  =    8'b1001__0010, 
        ED_ADCsHL_REG  =    8'b1001__1010, 
        ED_LDs6NN7_REG =    8'b1001__0011, 
        ED_LDsREG_6NN7 =    8'b1001__1011, 
        ED_NEG         =    7'b1001___100, 
        ED_RETN        =    7'b1001___101, 
        ED_RETI        =  10'h24D, 
        
        DBL_REG_BC   = 2'b00,  
        DBL_REG_DE   = 2'b01,  
        DBL_REG_HL   = 2'b10,  
        DBL_REG_SP   = 2'b11,  

        REG8_B = 3'b000,
        REG8_C = 3'b001,
        REG8_D = 3'b010,
        REG8_E = 3'b011,
        REG8_H = 3'b100,
        REG8_L = 3'b101,
        REG8_MEM = 3'b110,
        REG8_A = 3'b111;

































 







 







          
          
`line 233 "F:\verilog\wb_z80\new_wb_z80\wb_z80\rtl\z80_memstate2.v"


                     
parameter   TAG_IO    = 2'b01,   
            TAG_INT   = 2'b10;   
            
parameter   IPIPE_NOP       = 4'b0000,   
            IPIPE_A2        = 4'b0001,   
            IPIPE_ENN       = 4'b0010,   
            IPIPE_ENNA2     = 4'b0011,
            IPIPE_EN2       = 4'b0100,
            IPIPE_EN2A2     = 4'b0101,
            IPIPE_ENNEN2    = 4'b0110,
            IPIPE_ENNEN2A2  = 4'b0111,
            IPIPE_EN1       = 4'b1000,
            IPIPE_EN1A2     = 4'b1001,
            IPIPE_BOGUS     = 4'b1010,  
            IPIPE_BOUS2     = 4'b1011,
            IPIPE_EN12      = 4'b1100,
            IPIPE_EN12A2    = 4'b1101,
            IPIPE_BOGUS3    = 4'b1110,
            IPIPE_BOGUS4    = 4'b1111;
            




parameter       DEC_IDLE      = 6'h00,            
                DEC_HALT      = 6'h01,             
                DEC_IF1       = 6'h02,                 
                DEC_IF2       = 6'h03,               
                DEC_IF2A      = 6'h04,               
                DEC_EXEC      = 6'h05,            
                DEC_CB        = 6'h06,           
                DEC_DDFD      = 6'h07,               
                DEC_ED        = 6'h08,             
                DEC_EDNN1     = 6'h09,              
                DEC_EDNN2     = 6'h0a,             
                DEC_EDRD1     = 6'h0b,              
                DEC_EDRD2     = 6'h0c,              
                DEC_EDWR      = 6'h0d,               
                DEC_EDBCP1    = 6'h0e,             
                DEC_EDBCP2    = 6'h0f,             
                DEC_EDBCP3    = 6'h10,             
                DEC_EDBIN1    = 6'h11, 
                DEC_EDBIN2    = 6'h12, 
                DEC_EDBIN3    = 6'h13, 
                DEC_EDBOUT1   = 6'h14,
                DEC_EDBOUT2   = 6'h15,
                DEC_EDBOUT3   = 6'h16,
                DEC_EDBMV1    = 6'h17, 
                DEC_EDBMV2    = 6'h18, 
                DEC_EDBMV3    = 6'h19, 
                DEC_N         = 6'h1a,
                DEC_NIN       = 6'h1b,    
                DEC_NN        = 6'h1c,
                DEC_NNCALL1   = 6'h1d,
                DEC_NNCALL2   = 6'h1e,
                DEC_NNOS1     = 6'h1f,  
                DEC_NNOS2     = 6'h20,  
                DEC_NNOS3     = 6'h21,  
                DEC_NNOF1     = 6'h22,  
                DEC_NNOF2     = 6'h23,  
                DEC_NNOF3     = 6'h24,  
                DEC_NNOF4     = 6'h25,  
                DEC_DDOS      = 6'h26,   
                DEC_DDOF      = 6'h27,   
                DEC_OF        = 6'h28,     
                DEC_POP       = 6'h29,    
                DEC_PUSH      = 6'h2a,   
                DEC_RMW       = 6'h2b,    
                DEC_RMW2      = 6'h2c,   
                DEC_CBM       = 6'h2d,  
                DEC_PFxCB     = 6'h2e,  
                DEC_PFxCB2    = 6'h2f, 
                DEC_PFxCB3    = 6'h30, 
                DEC_PFxCB4    = 6'h31, 
                DEC_INT1      = 6'h32,   
                DEC_INT2      = 6'h33,   
                DEC_INT3      = 6'h34,   
                DEC_INT4      = 6'h35,   
                DEC_INT5      = 6'h36, 
                DEC_RET       = 6'h37,
                DEC_NNJMP     = 6'h38,
                DEC_DDN       = 6'h39,
                DEC_RET2      = 6'h3a,
                DEC_EXSPHL    = 6'h3b,
                DEC_RMWDD1    = 6'h3c,
                DEC_RMWDD2    = 6'h3d, 
                DEC_INT6      = 6'h3e ;


parameter      I1_CB    = 4'h0,
               I1_DDFD  = 4'h1,
               I1_ED    = 4'h2,
               I1_JMP   = 4'h3,
               I1_N     = 4'h4,
               I1_NN    = 4'h5,
               I1_OF    = 4'h6,
               I1_OS    = 4'h7,
               I1_POP   = 4'h8,
               I1_PUSH  = 4'h9,
               I1_RET   = 4'ha,
               I1_RMW   = 4'hb,
               I1_RST   = 4'hc,
               I1_R2R   = 4'hd,
               I1_JMPR  = 4'he,
               I1_HALT  = 4'hf;






parameter       MEM_NOP      = 5'h00,                                         
                MEM_IFPP1    = 5'h01,
                MEM_OS1      = 5'h02,      
                MEM_OF1      = 5'h03,     
                MEM_OFSP     = 5'h04,     
                MEM_OSSP     = 5'h05,     
                                          
                MEM_OFIXpD   = 5'h06,     
                MEM_OSIXpD   = 5'h07,     
                MEM_OSADR    = 5'h08,     
                
                MEM_CALL     = 5'h09,     
                MEM_OSNN     = 5'h0a,     
                MEM_OFNN     = 5'h0b,     
                MEM_OFADRP1  = 5'h0c,     
                MEM_OSADRP1  = 5'h0d,     
                             
                MEM_RST     = 5'h0e,     
                MEM_REL2PC  = 5'h0f,     
                MEM_JMPHL    = 5'h10,     
                MEM_IFNN     = 5'h11,        
                
                
                MEM_OFHL_PM  = 5'h12,             
                MEM_OSHL_PM  = 5'h13,             
                MEM_OSDE_PM  = 5'h14,             
                             
                MEM_IOF_C    = 5'h15,             
                MEM_IOS_C    = 5'h16,             
                MEM_IOF_N    = 5'h17,
                MEM_IOS_N    = 5'h18,
                MEM_OS_HL_N  = 5'h19,
                             
                MEM_OSSP_PCM2 = 5'h1a,              
                
                MEM_INTA     = 5'h1c,
                MEM_IFINT    = 5'h1d,
                MEM_DECPC    = 5'h1e ;             
                              

















wire        cb_mem;


                    


wire        wb_rdy_nhz;        
wire        dec_blk_inc;        
wire        we_next;
wire        hazard;
wire        wb_int;
wire [15:0] hl, de, bc;
wire [3:0]  mem_exec_dec;












wire  [5:0]        next_dec_state;
wire  [4:0]        next_mem_state;
wire  [3:0]        next_pipe_state;             
wire               ed_dbl_rd;  
wire  [15:0]       hl_or_ixiy;


reg [15:0]   pc;
reg [15:0]   sp;
reg [15:0]   wb_adr_o;
reg          wb_we_o; 
reg          wb_cyc_o; 
reg          wb_stb_o; 

reg [1:0]    wb_tga_o;

reg          blk_inc_flg;
reg [9:0]    ir1, ir2;
reg          ir1dd, ir2dd;
reg          ir1fd, ir2fd;
reg [15:0]   nn;

reg   [14:0]       next_state;      

reg   [5:0]  dec_state;    




reg          inst_haz;
reg          exec_ir2; 
reg          blk_rpt_flg;
reg          blk_io_flg;
reg          flag_os1;
reg          int_en, en_int_next;
reg          wb_irq_sync;
reg          ex_tos_hl;    




assign  blk_mv_upd_hl = next_mem_state == MEM_OFHL_PM & wb_rdy_nhz|
                        next_mem_state == MEM_OSHL_PM & wb_rdy_nhz ;
assign  blk_mv_upd_de = next_mem_state == MEM_OSDE_PM & wb_rdy_nhz;


assign  exec_decbc =  (dec_state == DEC_ED) & (  ed_blk_mv) & wb_rdy_nhz |
                      (dec_state == DEC_EDBMV3)   & wb_rdy_nhz           ;

assign  exec_decb = (dec_state == DEC_ED) & ( ed_blk_in | ed_blk_out) & wb_rdy_nhz|
                    (dec_state == DEC_EDBIN3)    & wb_rdy_nhz                     |
                    (dec_state == DEC_EDBOUT3)   & wb_rdy_nhz                      ;
assign wb_dat_o = nn[15:8];

wire   sf, zf, f5f, hf, f3f, pvf, nf, cf;
assign { sf, zf, f5f, hf, f3f, pvf, nf, cf} = fr;  


assign hl = {hr, lr};
assign de = {dr, er}; 
assign bc = {br, cr};

assign hl_or_ixiy = ir1dd ? ixr :
                    ir1fd ? iyr :
                            hl   ;


parameter  I1DCNT = 4;  
assign mem_exec_dec =
    {I1DCNT {CBgrp        == ir1}} & I1_CB  |
    {I1DCNT {DDgrp        == ir1}} & I1_DDFD|
    {I1DCNT {FDgrp        == ir1}} & I1_DDFD|
    {I1DCNT {EDgrp        == ir1}} & I1_ED  |
    {I1DCNT {JPsHL        == ir1}} & I1_JMP |
    {I1DCNT {ADCsA_N      == ir1}} & I1_N   |
    {I1DCNT {ADDsA_N      == ir1}} & I1_N   |
    {I1DCNT {ANDsN        == ir1}} & I1_N   |
    {I1DCNT {CPsN         == ir1}} & I1_N   |
    {I1DCNT {INsA_6N7     == ir1}} & I1_N   |
    {I1DCNT {JRs$t2       == ir1}} & I1_JMPR|
    {I1DCNT {JRsC_$t2     == ir1}} & I1_JMPR|
    {I1DCNT {JRsNC_$t2    == ir1}} & I1_JMPR|
    {I1DCNT {JRsZ_$t2     == ir1}} & I1_JMPR|
    {I1DCNT {JRsNZ_$t2    == ir1}} & I1_JMPR|
    {I1DCNT {LDs6HL7_N    == ir1}} & I1_N   |
    {I1DCNT {LDsA_N       == ir1}} & I1_N   |
    {I1DCNT {LDsB_N       == ir1}} & I1_N   |
    {I1DCNT {LDsC_N       == ir1}} & I1_N   |
    {I1DCNT {LDsD_N       == ir1}} & I1_N   |
    {I1DCNT {LDsE_N       == ir1}} & I1_N   |
    {I1DCNT {LDsH_N       == ir1}} & I1_N   |
    {I1DCNT {LDsL_N       == ir1}} & I1_N   |
    {I1DCNT {ORsN         == ir1}} & I1_N   |
    {I1DCNT {OUTs6N7_A    == ir1}} & I1_N   |
    {I1DCNT {SBCsA_N      == ir1}} & I1_N   |
    {I1DCNT {SUBsN        == ir1}} & I1_N   |
    {I1DCNT {XORsN        == ir1}} & I1_N   |
    {I1DCNT {CALLsC_NN    == ir1}} & I1_NN  |
    {I1DCNT {CALLsNC_NN   == ir1}} & I1_NN  |
    {I1DCNT {CALLsNN      == ir1}} & I1_NN  |
    {I1DCNT {CALLsNZ_NN   == ir1}} & I1_NN  |
    {I1DCNT {CALLsPE_NN   == ir1}} & I1_NN  |
    {I1DCNT {CALLsPO_NN   == ir1}} & I1_NN  |
    {I1DCNT {CALLsP_NN    == ir1}} & I1_NN  |
    {I1DCNT {CALLsZ_NN    == ir1}} & I1_NN  |
    {I1DCNT {CALLsM_NN    == ir1}} & I1_NN  |
    {I1DCNT {JP           == ir1}} & I1_NN  |
    {I1DCNT {JPsC         == ir1}} & I1_NN  |
    {I1DCNT {JPsM         == ir1}} & I1_NN  |
    {I1DCNT {JPsNC        == ir1}} & I1_NN  |
    {I1DCNT {JPsNZ        == ir1}} & I1_NN  |
    {I1DCNT {JPsP         == ir1}} & I1_NN  |
    {I1DCNT {JPsPE        == ir1}} & I1_NN  |
    {I1DCNT {JPsPO        == ir1}} & I1_NN  |
    {I1DCNT {JPsZ         == ir1}} & I1_NN  |
    {I1DCNT {LDs6NN7_A    == ir1}} & I1_NN  |
    {I1DCNT {LDs6NN7_HL   == ir1}} & I1_NN  |
    {I1DCNT {LDsA_6NN7    == ir1}} & I1_NN  |
    {I1DCNT {LDsBC_NN     == ir1}} & I1_NN  |
    {I1DCNT {LDsDE_NN     == ir1}} & I1_NN  |
    {I1DCNT {LDsHL_6NN7   == ir1}} & I1_NN  |
    {I1DCNT {LDsHL_NN     == ir1}} & I1_NN  |
    {I1DCNT {LDsSP_NN     == ir1}} & I1_NN  |
    {I1DCNT {ADCsA_6HL7   == ir1}} & I1_OF  |
    {I1DCNT {ADDsA_6HL7   == ir1}} & I1_OF  |
    {I1DCNT {ANDs6HL7     == ir1}} & I1_OF  |
    {I1DCNT {CPs6HL7      == ir1}} & I1_OF  |
    {I1DCNT {LDsA_6BC7    == ir1}} & I1_OF  |
    {I1DCNT {LDsA_6DE7    == ir1}} & I1_OF  |
    {I1DCNT {LDsA_6HL7    == ir1}} & I1_OF  |
    {I1DCNT {LDsB_6HL7    == ir1}} & I1_OF  |
    {I1DCNT {LDsC_6HL7    == ir1}} & I1_OF  |
    {I1DCNT {LDsD_6HL7    == ir1}} & I1_OF  |
    {I1DCNT {LDsE_6HL7    == ir1}} & I1_OF  |
    {I1DCNT {LDsH_6HL7    == ir1}} & I1_OF  |
    {I1DCNT {LDsL_6HL7    == ir1}} & I1_OF  |
    {I1DCNT {ORs6HL7      == ir1}} & I1_OF  |
    {I1DCNT {SBCs6HL7     == ir1}} & I1_OF  |
    {I1DCNT {SUBs6HL7     == ir1}} & I1_OF  |
    {I1DCNT {XORs6HL7     == ir1}} & I1_OF  |
    {I1DCNT {LDs6BC7_A    == ir1}} & I1_OS  |
    {I1DCNT {LDs6DE7_A    == ir1}} & I1_OS  |
    {I1DCNT {LDs6HL7_A    == ir1}} & I1_OS  |
    {I1DCNT {LDs6HL7_B    == ir1}} & I1_OS  |
    {I1DCNT {LDs6HL7_C    == ir1}} & I1_OS  |
    {I1DCNT {LDs6HL7_D    == ir1}} & I1_OS  |
    {I1DCNT {LDs6HL7_E    == ir1}} & I1_OS  |
    {I1DCNT {LDs6HL7_H    == ir1}} & I1_OS  |
    {I1DCNT {LDs6HL7_L    == ir1}} & I1_OS  |
    {I1DCNT {POPsAF       == ir1}} & I1_POP |
    {I1DCNT {POPsBC       == ir1}} & I1_POP |
    {I1DCNT {POPsDE       == ir1}} & I1_POP |
    {I1DCNT {POPsHL       == ir1}} & I1_POP |
    {I1DCNT {PUSHsAF      == ir1}} & I1_PUSH|
    {I1DCNT {PUSHsBC      == ir1}} & I1_PUSH|
    {I1DCNT {PUSHsDE      == ir1}} & I1_PUSH|
    {I1DCNT {PUSHsHL      == ir1}} & I1_PUSH|
    {I1DCNT {ADCsA_A      == ir1}} & I1_R2R |
    {I1DCNT {ADCsA_B      == ir1}} & I1_R2R |
    {I1DCNT {ADCsA_C      == ir1}} & I1_R2R |
    {I1DCNT {ADCsA_D      == ir1}} & I1_R2R |
    {I1DCNT {ADCsA_E      == ir1}} & I1_R2R |
    {I1DCNT {ADCsA_H      == ir1}} & I1_R2R |
    {I1DCNT {ADCsA_L      == ir1}} & I1_R2R |
    {I1DCNT {ADDsA_A      == ir1}} & I1_R2R |
    {I1DCNT {ADDsA_B      == ir1}} & I1_R2R |
    {I1DCNT {ADDsA_C      == ir1}} & I1_R2R |
    {I1DCNT {ADDsA_D      == ir1}} & I1_R2R |
    {I1DCNT {ADDsA_E      == ir1}} & I1_R2R |
    {I1DCNT {ADDsA_H      == ir1}} & I1_R2R |
    {I1DCNT {ADDsA_L      == ir1}} & I1_R2R |
    {I1DCNT {ADDsHL_BC    == ir1}} & I1_R2R |
    {I1DCNT {ADDsHL_DE    == ir1}} & I1_R2R |
    {I1DCNT {ADDsHL_HL    == ir1}} & I1_R2R |
    {I1DCNT {ADDsHL_SP    == ir1}} & I1_R2R |
    {I1DCNT {ANDsA        == ir1}} & I1_R2R |
    {I1DCNT {ANDsB        == ir1}} & I1_R2R |
    {I1DCNT {ANDsC        == ir1}} & I1_R2R |
    {I1DCNT {ANDsD        == ir1}} & I1_R2R |
    {I1DCNT {ANDsE        == ir1}} & I1_R2R |
    {I1DCNT {ANDsH        == ir1}} & I1_R2R |
    {I1DCNT {ANDsL        == ir1}} & I1_R2R |
    {I1DCNT {CCF          == ir1}} & I1_R2R |
    {I1DCNT {CPL          == ir1}} & I1_R2R |
    {I1DCNT {CPsA         == ir1}} & I1_R2R |
    {I1DCNT {CPsB         == ir1}} & I1_R2R |
    {I1DCNT {CPsC         == ir1}} & I1_R2R |
    {I1DCNT {CPsD         == ir1}} & I1_R2R |
    {I1DCNT {CPsE         == ir1}} & I1_R2R |
    {I1DCNT {CPsH         == ir1}} & I1_R2R |
    {I1DCNT {CPsL         == ir1}} & I1_R2R |
    {I1DCNT {DAA          == ir1}} & I1_R2R |
    {I1DCNT {DECsA        == ir1}} & I1_R2R |
    {I1DCNT {DECsB        == ir1}} & I1_R2R |
    {I1DCNT {DECsBC       == ir1}} & I1_R2R |
    {I1DCNT {DECsC        == ir1}} & I1_R2R |
    {I1DCNT {DECsD        == ir1}} & I1_R2R |
    {I1DCNT {DECsDE       == ir1}} & I1_R2R |
    {I1DCNT {DECsE        == ir1}} & I1_R2R |
    {I1DCNT {DECsH        == ir1}} & I1_R2R |
    {I1DCNT {DECsHL       == ir1}} & I1_R2R |
    {I1DCNT {DECsL        == ir1}} & I1_R2R |
    {I1DCNT {DECsSP       == ir1}} & I1_R2R |
    {I1DCNT {DI           == ir1}} & I1_R2R |
    {I1DCNT {DJNZs$t2     == ir1}} & I1_JMPR|
    {I1DCNT {EI           == ir1}} & I1_R2R |
    {I1DCNT {EXX          == ir1}} & I1_R2R |
    {I1DCNT {EXsAF_AFp    == ir1}} & I1_R2R |
    {I1DCNT {EXsDE_HL     == ir1}} & I1_R2R |
    {I1DCNT {HALT         == ir1}} & I1_HALT |
    {I1DCNT {INCsA        == ir1}} & I1_R2R |
    {I1DCNT {INCsB        == ir1}} & I1_R2R |
    {I1DCNT {INCsBC       == ir1}} & I1_R2R |
    {I1DCNT {INCsC        == ir1}} & I1_R2R |
    {I1DCNT {INCsD        == ir1}} & I1_R2R |
    {I1DCNT {INCsDE       == ir1}} & I1_R2R |
    {I1DCNT {INCsE        == ir1}} & I1_R2R |
    {I1DCNT {INCsH        == ir1}} & I1_R2R |
    {I1DCNT {INCsHL       == ir1}} & I1_R2R |
    {I1DCNT {INCsL        == ir1}} & I1_R2R |
    {I1DCNT {INCsSP       == ir1}} & I1_R2R |
    {I1DCNT {LDsA_A       == ir1}} & I1_R2R |
    {I1DCNT {LDsA_B       == ir1}} & I1_R2R |
    {I1DCNT {LDsA_C       == ir1}} & I1_R2R |
    {I1DCNT {LDsA_D       == ir1}} & I1_R2R |
    {I1DCNT {LDsA_E       == ir1}} & I1_R2R |
    {I1DCNT {LDsA_H       == ir1}} & I1_R2R |
    {I1DCNT {LDsA_L       == ir1}} & I1_R2R |
    {I1DCNT {LDsB_A       == ir1}} & I1_R2R |
    {I1DCNT {LDsB_B       == ir1}} & I1_R2R |
    {I1DCNT {LDsB_C       == ir1}} & I1_R2R |
    {I1DCNT {LDsB_D       == ir1}} & I1_R2R |
    {I1DCNT {LDsB_E       == ir1}} & I1_R2R |
    {I1DCNT {LDsB_H       == ir1}} & I1_R2R |
    {I1DCNT {LDsB_L       == ir1}} & I1_R2R |
    {I1DCNT {LDsC_A       == ir1}} & I1_R2R |
    {I1DCNT {LDsC_B       == ir1}} & I1_R2R |
    {I1DCNT {LDsC_C       == ir1}} & I1_R2R |
    {I1DCNT {LDsC_D       == ir1}} & I1_R2R |
    {I1DCNT {LDsC_E       == ir1}} & I1_R2R |
    {I1DCNT {LDsC_H       == ir1}} & I1_R2R |
    {I1DCNT {LDsC_L       == ir1}} & I1_R2R |
    {I1DCNT {LDsD_A       == ir1}} & I1_R2R |
    {I1DCNT {LDsD_B       == ir1}} & I1_R2R |
    {I1DCNT {LDsD_C       == ir1}} & I1_R2R |
    {I1DCNT {LDsD_D       == ir1}} & I1_R2R |
    {I1DCNT {LDsD_E       == ir1}} & I1_R2R |
    {I1DCNT {LDsD_H       == ir1}} & I1_R2R |
    {I1DCNT {LDsD_L       == ir1}} & I1_R2R |
    {I1DCNT {LDsE_A       == ir1}} & I1_R2R |
    {I1DCNT {LDsE_B       == ir1}} & I1_R2R |
    {I1DCNT {LDsE_C       == ir1}} & I1_R2R |
    {I1DCNT {LDsE_D       == ir1}} & I1_R2R |
    {I1DCNT {LDsE_E       == ir1}} & I1_R2R |
    {I1DCNT {LDsE_H       == ir1}} & I1_R2R |
    {I1DCNT {LDsE_L       == ir1}} & I1_R2R |
    {I1DCNT {LDsH_A       == ir1}} & I1_R2R |
    {I1DCNT {LDsH_B       == ir1}} & I1_R2R |
    {I1DCNT {LDsH_C       == ir1}} & I1_R2R |
    {I1DCNT {LDsH_D       == ir1}} & I1_R2R |
    {I1DCNT {LDsH_E       == ir1}} & I1_R2R |
    {I1DCNT {LDsH_H       == ir1}} & I1_R2R |
    {I1DCNT {LDsH_L       == ir1}} & I1_R2R |
    {I1DCNT {LDsL_A       == ir1}} & I1_R2R |
    {I1DCNT {LDsL_B       == ir1}} & I1_R2R |
    {I1DCNT {LDsL_C       == ir1}} & I1_R2R |
    {I1DCNT {LDsL_D       == ir1}} & I1_R2R |
    {I1DCNT {LDsL_E       == ir1}} & I1_R2R |
    {I1DCNT {LDsL_H       == ir1}} & I1_R2R |
    {I1DCNT {LDsL_L       == ir1}} & I1_R2R |
    {I1DCNT {LDsSP_HL     == ir1}} & I1_R2R |
    {I1DCNT {NOP          == ir1}} & I1_R2R |
    {I1DCNT {ORsA         == ir1}} & I1_R2R |
    {I1DCNT {ORsB         == ir1}} & I1_R2R |
    {I1DCNT {ORsC         == ir1}} & I1_R2R |
    {I1DCNT {ORsD         == ir1}} & I1_R2R |
    {I1DCNT {ORsE         == ir1}} & I1_R2R |
    {I1DCNT {ORsH         == ir1}} & I1_R2R |
    {I1DCNT {ORsL         == ir1}} & I1_R2R |
    {I1DCNT {RLA          == ir1}} & I1_R2R |
    {I1DCNT {RLCA         == ir1}} & I1_R2R |
    {I1DCNT {RRA          == ir1}} & I1_R2R |
    {I1DCNT {RRCA         == ir1}} & I1_R2R |
    {I1DCNT {SBCsA        == ir1}} & I1_R2R |
    {I1DCNT {SBCsB        == ir1}} & I1_R2R |
    {I1DCNT {SBCsC        == ir1}} & I1_R2R |
    {I1DCNT {SBCsD        == ir1}} & I1_R2R |
    {I1DCNT {SBCsE        == ir1}} & I1_R2R |
    {I1DCNT {SBCsH        == ir1}} & I1_R2R |
    {I1DCNT {SBCsL        == ir1}} & I1_R2R |
    {I1DCNT {SCF          == ir1}} & I1_R2R |
    {I1DCNT {SUBsA        == ir1}} & I1_R2R |
    {I1DCNT {SUBsB        == ir1}} & I1_R2R |
    {I1DCNT {SUBsC        == ir1}} & I1_R2R |
    {I1DCNT {SUBsD        == ir1}} & I1_R2R |
    {I1DCNT {SUBsE        == ir1}} & I1_R2R |
    {I1DCNT {SUBsH        == ir1}} & I1_R2R |
    {I1DCNT {SUBsL        == ir1}} & I1_R2R |
    {I1DCNT {XORsA        == ir1}} & I1_R2R |
    {I1DCNT {XORsB        == ir1}} & I1_R2R |
    {I1DCNT {XORsC        == ir1}} & I1_R2R |
    {I1DCNT {XORsD        == ir1}} & I1_R2R |
    {I1DCNT {XORsE        == ir1}} & I1_R2R |
    {I1DCNT {XORsH        == ir1}} & I1_R2R |
    {I1DCNT {XORsL        == ir1}} & I1_R2R |
    {I1DCNT {RET          == ir1}} & I1_RET |
    {I1DCNT {RETsC == ir1 & cf  }} & I1_RET |
    {I1DCNT {RETsM == ir1 & sf  }} & I1_RET |
    {I1DCNT {RETsNC== ir1 & ~cf }} & I1_RET |
    {I1DCNT {RETsP == ir1 & ~sf }} & I1_RET |
    {I1DCNT {RETsPE== ir1 & pvf }} & I1_RET |
    {I1DCNT {RETsPO== ir1 & ~pvf}} & I1_RET |
    {I1DCNT {RETsNZ== ir1 & ~zf }} & I1_RET |
    {I1DCNT {RETsZ == ir1 & zf  }} & I1_RET |
    {I1DCNT {EXs6SP7_HL   == ir1}} & I1_POP |
    {I1DCNT {DECs6HL7     == ir1}} & I1_RMW |
    {I1DCNT {INCs6HL7     == ir1}} & I1_RMW |
    {I1DCNT {RSTs0        == ir1}} & I1_RST |
    {I1DCNT {RSTs10H      == ir1}} & I1_RST |
    {I1DCNT {RSTs18H      == ir1}} & I1_RST |
    {I1DCNT {RSTs20H      == ir1}} & I1_RST |
    {I1DCNT {RSTs28H      == ir1}} & I1_RST |
    {I1DCNT {RSTs30H      == ir1}} & I1_RST |
    {I1DCNT {RSTs38H      == ir1}} & I1_RST |
    {I1DCNT {RSTs8H       == ir1}} & I1_RST ;
                











           





assign cb_mem =  (CB_MEM  == ir1[2:0]);   














           











wire dec_blk_rpt =            
           ED_LDIR      == ir1 |
           ED_CPIR      == ir1 |
           ED_INIR      == ir1 |
           ED_OTIR      == ir1 |
           ED_LDDR      == ir1 |
           ED_CPDR      == ir1 |
           ED_INDR      == ir1 |
           ED_OTDR      == ir1 ;
wire ed_blk_mv =  ED_LDIR      == ir1 |  ED_LDI       == ir1 |
                  ED_LDDR      == ir1 |  ED_LDD       == ir1 ;
wire ed_blk_cp =  ED_CPIR      == ir1 |  ED_CPI       == ir1 |
                  ED_CPDR      == ir1 |  ED_CPD       == ir1 ;
wire ed_blk_in =  ED_INIR      == ir1 |  ED_INI      == ir1 |
                  ED_INDR      == ir1 |  ED_IND      == ir1 ;

wire ed_blk_out = ED_OTIR      == ir1 |  ED_OUTI      == ir1 |
                  ED_OTDR      == ir1 |  ED_OUTD      == ir1 ;

wire dec_blk_io = ed_blk_in | ed_blk_out;

wire blk_done =  ~blk_rpt_flg |  beq0 & ceq0 | blk_io_flg & beq0;
wire blk_cpi_done = ~blk_rpt_flg | ({br, cr} == 16'h1);
assign dec_blk_inc =  ED_LDIR      == ir1 |
                      ED_CPIR      == ir1 |
                      ED_INIR      == ir1 |
                      ED_OTIR      == ir1 |
                      ED_LDI       == ir1 |
                      ED_CPI       == ir1 |
                      ED_INI       == ir1 |
                      ED_OUTI      == ir1 ;



















wire ed_in_reg  = ED_INsREG_6C7   ==  {ir1[9:6],ir1[2:0]}; 
wire ed_out_reg = ED_OUTs6C7_REG  ==  {ir1[9:6],ir1[2:0]}; 

wire ed_nn = ED_LDs6NN7_REG == {ir1[9:6],ir1[3:0]} |
             ED_LDsREG_6NN7 == {ir1[9:6],ir1[3:0]}  ;


wire ed_retn = ED_RETN == {ir1[9:6],ir1[2:0]};
wire ed_rmw = ED_RRD == ir1 |  
              ED_RLD == ir1  ; 

assign ed_dbl_rd =  ED_LDsREG_6NN7 == {ir1[9:6],ir1[3:0]};   
    



            


wire jmpr_true =               
    JRs$t2       == ir1           |    
    JRsC_$t2     == ir1  & fr[0]  |            
    JRsNC_$t2    == ir1  & ~fr[0] |            
    JRsZ_$t2     == ir1  & fr[6]  |             
    JRsNZ_$t2    == ir1  & ~fr[6] |
    DJNZs$t2     == ir1  & (br != 8'h1); 
wire jmpr =               
    JRs$t2       == ir1   |    
    JRsC_$t2     == ir1   |            
    JRsNC_$t2    == ir1   |            
    JRsZ_$t2     == ir1   |             
    JRsNZ_$t2    == ir1   |   
    DJNZs$t2     == ir1;     
    
              

wire callnn_true   =  CALLsC_NN    == ir1  & cf  |
                      CALLsNC_NN   == ir1  & ~cf |
                      CALLsNN      == ir1        |
                      CALLsNZ_NN   == ir1  & ~zf |
                      CALLsPE_NN   == ir1  & pvf | 
                      CALLsPO_NN   == ir1  & ~pvf|
                      CALLsP_NN    == ir1  & ~sf |
                      CALLsZ_NN    == ir1  &  zf |  
                      CALLsM_NN    == ir1  &  sf  ;

wire  jmpnn_true  =  JPsC         == ir1  & cf  |
                     JPsNC        == ir1  & ~cf |
                     JP           == ir1        |                   
                     JPsNZ        == ir1  & ~zf |
                     JPsPE        == ir1  & pvf | 
                     JPsPO        == ir1  & ~pvf|
                     JPsP         == ir1  & ~sf |
                     JPsZ         == ir1  &  zf | 
                     JPsM         == ir1  &  sf  ;






wire os_a  =  LDs6BC7_A    == ir1 |  
              LDs6DE7_A    == ir1 |  
              LDs6HL7_A    == ir1 |  
              LDs6NN7_A    == ir1 |  
              PUSHsAF      == ir1 |
              OUTs6N7_A    == ir1 |
              (ED_OUTs6C7_REG ==  {ir1[9:6],ir1[2:0]}) & REG8_A == ir1[5:3] ;

wire os_b = LDs6HL7_B      == ir1                                       |  
            ED_LDs6NN7_REG == {ir1[9:6],ir1[3:0]} & DBL_REG_BC == ir1[5:4] |
            PUSHsBC        == ir1                                       |  
            ED_OUTs6C7_REG ==  {ir1[9:6],ir1[2:0]} & REG8_B == ir1[5:3] ;   
            
wire os_c = LDs6HL7_C    == ir1                                         |  
            ED_OUTs6C7_REG ==  {ir1[9:6],ir1[2:0]} & REG8_C == ir1[5:3] ;
            
wire os_d = LDs6HL7_D    == ir1                                         |  
            PUSHsDE      == ir1                                         |  
            ED_LDs6NN7_REG == {ir1[9:6],ir1[3:0]} & DBL_REG_DE == ir1[5:4] |
            ED_OUTs6C7_REG ==  {ir1[9:6],ir1[2:0]} & REG8_D == ir1[5:3] ;
            
            
wire os_e = LDs6HL7_E    == ir1                                     |  
            ED_OUTs6C7_REG ==  {ir1[9:6],ir1[2:0]} & REG8_E == ir1[5:3] ;
            
wire os_h = LDs6HL7_H    == ir1                                         |  
            PUSHsHL      == ir1                                         |  
            ED_OUTs6C7_REG ==  {ir1[9:6],ir1[2:0]} & REG8_H == ir1[5:3] ;

wire os_l = LDs6HL7_L    == ir1                                     |  
            ED_OUTs6C7_REG ==  {ir1[9:6],ir1[2:0]} & REG8_L == ir1[5:3] ;




wire os_bc = ED_LDs6NN7_REG == {ir1[9:6],ir1[3:0]} & DBL_REG_BC == ir1[5:4];
wire os_de = ED_LDs6NN7_REG == {ir1[9:6],ir1[3:0]} & DBL_REG_DE == ir1[5:4];
wire os_sp = ED_LDs6NN7_REG == {ir1[9:6],ir1[3:0]} & DBL_REG_SP == ir1[5:4];


wire os_hl =   LDs6NN7_HL   == ir1 & ~(ir1dd | ir2dd)                        | 
               ED_LDs6NN7_REG == {ir1[9:6],ir1[3:0]} & DBL_REG_HL == ir1[5:4] ;
               
wire os_ixr =  LDs6NN7_HL   == ir1 & ir1dd;
wire os_iyr =  LDs6NN7_HL   == ir1 & ir1fd;

























                                        





















wire  use_hl_exec =  LDsSP_HL == ir1      |  INCs6HL7     == ir1  |
                     ADCsA_6HL7   == ir1  |  SBCs6HL7     == ir1  |
                     ADDsA_6HL7   == ir1  |  SUBs6HL7     == ir1  |
                     ANDs6HL7     == ir1  |  XORs6HL7     == ir1  |
                     CPs6HL7      == ir1  |  LDs6HL7_A    == ir1  |
                     LDsA_6HL7    == ir1  |  LDs6HL7_B    == ir1  |
                     LDsB_6HL7    == ir1  |  LDs6HL7_C    == ir1  |
                     LDsC_6HL7    == ir1  |  LDs6HL7_D    == ir1  |
                     LDsD_6HL7    == ir1  |  LDs6HL7_E    == ir1  |
                     LDsE_6HL7    == ir1  |  LDs6HL7_H    == ir1  |
                     LDsH_6HL7    == ir1  |  LDs6HL7_L    == ir1  |
                     LDsL_6HL7    == ir1  |  JPsHL        == ir1  |
                     ORs6HL7      == ir1  |  DECs6HL7     == ir1  |
                     PUSHsHL      == ir1;
wire  use_bc_exec =  LDsA_6BC7    == ir1 |
                     LDs6BC7_A    == ir1 |
                     PUSHsBC      == ir1  ;
wire  use_de_exec =  LDs6DE7_A    == ir1 |
                     LDsA_6DE7    == ir1 |
                     PUSHsDE      == ir1  ;
                     
wire  use_sp_exec =  MEM_OFSP == next_mem_state |
                     MEM_OSSP == next_mem_state  ; 
wire  upd_sp_exec  = DECsSP == ir2 |
                     INCsSP == ir2   ;

wire use_fr_exec = ( RETsC        == ir1  |
                     RETsM        == ir1  |
                     RETsNC       == ir1  |
                     RETsP        == ir1  |
                     RETsPE       == ir1  |
                     RETsPO       == ir1  |
                     RETsNZ       == ir1  |
                     RETsZ        == ir1  |
                     PUSHsAF      == ir1   ) ;

assign hazard =  (dec_state == DEC_EXEC  & exec_ir2 ) & ( upd_fr & use_fr_exec  |
                                                          upd_ar & os_a         | 
                                                          upd_br & os_b         |
                                                          upd_br & use_bc_exec  |
                                                          upd_cr & os_c         |
                                                          upd_cr & use_bc_exec  |
                                                          upd_dr & os_d         |
                                                          upd_dr & use_de_exec  |
                                                          upd_er & os_e         |
                                                          upd_er & use_de_exec  |
                                                          upd_hr & os_h         | 
                                                          upd_lr & os_l         |
                                                          upd_hr & use_hl_exec  |
                                                          upd_lr & use_hl_exec  |
                                                          upd_sp_exec & use_sp_exec );









wire  opadr_bc  =  LDsA_6BC7  == ir1 | LDs6BC7_A == ir1;
wire  opadr_de  =  LDsA_6DE7  == ir1 | LDs6DE7_A == ir1;
wire  opadr_hl  =  LDsB_6HL7  == ir1 | ORs6HL7    == ir1 | LDs6HL7_B == ir1 |
                   LDsD_6HL7  == ir1 | LDsC_6HL7  == ir1 | LDs6HL7_C == ir1 |
                   LDsH_6HL7  == ir1 | LDsE_6HL7  == ir1 | LDs6HL7_D == ir1 |
                   ADDsA_6HL7 == ir1 | LDsL_6HL7  == ir1 | LDs6HL7_E == ir1 |
                   SUBs6HL7   == ir1 | LDsA_6HL7  == ir1 | LDs6HL7_H == ir1 |
                   ANDs6HL7   == ir1 | ADCsA_6HL7 == ir1 | LDs6HL7_L == ir1 |
                   XORs6HL7   == ir1 | SBCs6HL7   == ir1 | CPs6HL7   == ir1 ;
                                       



















                    
                    
                    
                     
                    
                    

























wire src_sp = next_mem_state == MEM_OFSP                     |
              next_mem_state == MEM_OSSP                     |
              next_mem_state == MEM_OSSP_PCM2                |
              next_mem_state == MEM_CALL                       ;
wire src_pc =  next_mem_state ==   MEM_IFPP1   |
               next_mem_state ==  MEM_REL2PC  ;

wire src_nn =  next_mem_state ==   MEM_IFNN | 
               next_mem_state ==   MEM_OSNN |
               next_mem_state ==   MEM_OFNN  ;
                                             

wire src_de  = dec_state == DEC_EXEC & LDsA_6DE7 == ir1  |      
               dec_state == DEC_EXEC & LDs6DE7_A == ir1  |     
               next_mem_state == MEM_OSDE_PM               ;
wire src_bc =  dec_state == DEC_EXEC & LDsA_6BC7 == ir1  |
               dec_state == DEC_EXEC & LDs6BC7_A == ir1  |
               next_mem_state ==MEM_IOF_C                |
               next_mem_state ==MEM_IOS_C                 ;





wire src_hl =   next_mem_state == MEM_OF1  & 
                                   !src_de & !src_bc & !src_sp  |
                next_mem_state == MEM_OS1  &                    
                                   !src_de & !src_bc         |                   
                next_mem_state == MEM_OFHL_PM                |
                next_mem_state == MEM_OSHL_PM                |
                next_mem_state == MEM_OS_HL_N                | 
                next_mem_state == MEM_JMPHL  & !( ir1dd | ir1fd);
                
wire src_ix =  next_mem_state == MEM_OFIXpD  &  ir1dd |
               next_mem_state == MEM_JMPHL   &  ir1dd |
               next_mem_state == MEM_OSIXpD  &  ir1dd  ;

wire src_iy =  next_mem_state == MEM_OFIXpD  &  ir1fd |
               next_mem_state == MEM_JMPHL   &  ir1fd |
               next_mem_state == MEM_OSIXpD  &  ir1fd  ;
               
wire src_adr = next_mem_state == MEM_OFADRP1  |
               next_mem_state == MEM_OSADRP1  |
               next_mem_state == MEM_NOP      |
               next_mem_state == MEM_OSADR     ;

wire src_io = next_mem_state == MEM_IOF_N  |
               next_mem_state == MEM_IOS_N   ;
               
wire src_int = next_mem_state == MEM_IFINT ;                                 

wire [15:0]  src_mux =   {16{ src_sp  }} & sp                 |
                         {16{ src_pc  }} & pc                 |
                         {16{ src_nn  }} & nn                 |
                         {16{ src_hl  }} & hl                 |
                         {16{ src_de  }} & de                 |
                         {16{ src_bc  }} & bc                 |
                         {16{ src_ix  }} & ixr                |
                         {16{ src_iy  }} & iyr                |
                         {16{ src_adr }} & wb_adr_o           |
                         {16{ src_int }} & {intr, nn[15:8] }  |
                         {16{ src_io }} & { br, nn[15:8] }  ;
                   
wire block_mv_inc = (dec_state == DEC_ED) ? dec_blk_inc : blk_inc_flg; 



wire inc    =     next_mem_state ==MEM_OFADRP1                |
                  next_mem_state ==MEM_OSADRP1                |
                  next_mem_state ==MEM_OFHL_PM & block_mv_inc |
                  next_mem_state ==MEM_OSHL_PM & block_mv_inc |
                  next_mem_state ==MEM_OSDE_PM & block_mv_inc |
                  next_mem_state ==MEM_OFSP                   |
                  next_mem_state ==MEM_IFPP1                  |
                  next_mem_state ==MEM_JMPHL                  |
                  next_mem_state ==MEM_IFINT                  |
                  next_mem_state ==MEM_IFNN                   |
                  next_mem_state ==MEM_OFNN                   |
                  next_mem_state ==MEM_OSNN                   |
                  next_mem_state ==MEM_RST                     ;
                  

wire dec    =     next_mem_state ==MEM_OFHL_PM & ~block_mv_inc |
                  next_mem_state ==MEM_OSHL_PM & ~block_mv_inc |
                  next_mem_state ==MEM_OSDE_PM & ~block_mv_inc |
                  next_mem_state ==MEM_OSSP_PCM2               |
                  next_mem_state ==MEM_CALL                    |
                  next_mem_state == MEM_OSSP                    ;
                  

wire reln    =    next_mem_state ==  MEM_REL2PC   | 
                  next_mem_state ==  MEM_OFIXpD    |
                   next_mem_state ==  MEM_OSIXpD    ;

wire  [15:0] src2    = {16{ inc    }}  & 16'h0001               |
                       {16{ dec    }}  & 16'hffff               |                 
                       {16{ reln    }}  & {{8{nn[15]}},nn[15:8]}|
                       {16{~(reln | inc | dec)}} & 16'h0    ;
                                                             
                                                             

wire [15:0]  adr_alu     = src2 + src_mux;                 
                  

wire  pre_inc_dec =    next_mem_state ==  MEM_CALL      | 
                       
                       next_mem_state ==  MEM_REL2PC    |
                       next_mem_state ==  MEM_OFIXpD    |
                       next_mem_state ==  MEM_OSIXpD    |
                       next_mem_state ==  MEM_OFADRP1   |
                       next_mem_state ==  MEM_OSADRP1   |
                       next_mem_state ==  MEM_OSSP_PCM2 |
                       next_mem_state ==  MEM_OSSP     ;


wire  [15:0] mux21 =  pre_inc_dec ? adr_alu : src_mux; 

assign wb_rdy_nhz = (!wb_cyc_o | wb_ack_i ) & ~hazard;   
wire   wb_rdy     = !wb_cyc_o | wb_ack_i;

assign we_next = next_mem_state == MEM_OS1        |
                 next_mem_state == MEM_OSSP       |
                 next_mem_state == MEM_OSIXpD     |
                 next_mem_state == MEM_OSADR      |
                 next_mem_state == MEM_OSSP_PCM2  |
                 
                 next_mem_state == MEM_CALL       |
                 next_mem_state == MEM_OSNN       |
                 next_mem_state == MEM_OSADRP1    |
                 next_mem_state == MEM_OSHL_PM    |
                 next_mem_state == MEM_OSDE_PM    |
                 next_mem_state == MEM_OS_HL_N    |
                 next_mem_state == MEM_IOS_C      |
                 next_mem_state == MEM_IOS_N       ;





assign {next_dec_state, next_mem_state, next_pipe_state} = next_state;

always @(ir1 or wb_int or inst_haz  or dec_state or mem_exec_dec or cb_mem or ed_nn or
         ed_blk_cp  or ed_blk_in or ed_blk_out or ed_retn or ed_blk_mv or ed_dbl_rd or blk_done or 
         fr or jmpr_true or callnn_true or jmpnn_true or 
         ed_rmw or ed_in_reg or blk_cpi_done or jmpr or ex_tos_hl)
         
begin
    case (dec_state)
        DEC_IDLE:       next_state = {DEC_IF1, MEM_NOP, IPIPE_NOP};
        
        DEC_HALT: 
            if (wb_int)      next_state = {DEC_INT1,MEM_NOP   ,IPIPE_NOP};
            else             next_state = {DEC_HALT,MEM_NOP   ,IPIPE_NOP};  
        DEC_IF1 :            next_state = {DEC_IF2 ,MEM_IFPP1 ,IPIPE_NOP};
        DEC_IF2 :            next_state = {DEC_EXEC,MEM_IFPP1 ,IPIPE_EN1};
        DEC_IF2A:            next_state = {DEC_EXEC,MEM_IFPP1 ,IPIPE_NOP};
        DEC_EXEC:
            if      (inst_haz)    next_state = {DEC_IF1, MEM_DECPC , IPIPE_NOP};
            else if (wb_int)      next_state = {DEC_INT1,MEM_NOP   ,IPIPE_NOP};
            else 
                case (mem_exec_dec) 
                I1_CB   : next_state = {DEC_CB,   MEM_IFPP1, IPIPE_EN1};
                I1_DDFD : next_state = {DEC_DDFD, MEM_IFPP1, IPIPE_EN1};
                I1_ED   : next_state = {DEC_ED,   MEM_IFPP1, IPIPE_EN1};
                I1_JMP  : next_state = {DEC_IF2,  MEM_JMPHL, IPIPE_NOP};   
                I1_N    : next_state = {DEC_N,    MEM_IFPP1, IPIPE_ENNEN2};
                I1_NN   : next_state = {DEC_NN,   MEM_IFPP1, IPIPE_ENN};
                I1_OF   : next_state = {DEC_OF,   MEM_OF1,   IPIPE_EN12};
                I1_OS   : next_state = {DEC_IF2A,  MEM_OS1,   IPIPE_EN1}; 
                I1_POP  : next_state = {DEC_POP,  MEM_OFSP,  IPIPE_EN12};
                I1_PUSH : next_state = {DEC_PUSH, MEM_OSSP,  IPIPE_EN12};
                I1_RET  : next_state = {DEC_RET,  MEM_OFSP,  IPIPE_EN12};
                I1_RMW  : next_state = {DEC_RMW,  MEM_OF1,   IPIPE_EN12};
                I1_RST  : next_state = {DEC_NNCALL2,  MEM_OSSP_PCM2, IPIPE_NOP};
                I1_R2R  : next_state = {DEC_EXEC, MEM_IFPP1, IPIPE_EN12A2};
                I1_JMPR : next_state = {DEC_N,    MEM_NOP,   IPIPE_ENNEN2A2};
                I1_HALT : next_state = {DEC_HALT, MEM_NOP  , IPIPE_EN2};
                default : next_state = {DEC_EXEC, MEM_IFPP1, IPIPE_EN12A2}; 
                endcase
        DEC_CB: if (cb_mem) next_state = {DEC_CBM, MEM_OF1, IPIPE_EN12};
                else        next_state = {DEC_EXEC, MEM_IFPP1, IPIPE_EN12A2};
        DEC_DDFD:   
            case (mem_exec_dec)
            I1_CB   : next_state = {DEC_PFxCB,MEM_IFPP1, IPIPE_ENN};
            I1_DDFD : next_state = {DEC_DDFD, MEM_IFPP1, IPIPE_EN1};     
            I1_ED   : next_state = {DEC_ED,   MEM_IFPP1, IPIPE_EN1};
            I1_JMP  : next_state = {DEC_IF2,  MEM_JMPHL, IPIPE_NOP};  
            I1_N    : next_state = {DEC_DDN,  MEM_IFPP1, IPIPE_ENN};
            I1_NN   : next_state = {DEC_NN,   MEM_IFPP1, IPIPE_ENN};
            I1_OF   : next_state = {DEC_DDOF, MEM_IFPP1, IPIPE_ENN};  
                                                                      
                                                                      
                                                                      
                                                                      
                                                                      
                                                                      
            I1_OS   : next_state = {DEC_DDOS, MEM_IFPP1, IPIPE_ENN};  
            I1_POP  : next_state = {DEC_POP,  MEM_OFSP,  IPIPE_EN12};
            I1_PUSH : next_state = {DEC_PUSH, MEM_OSSP,  IPIPE_EN12};
            I1_RET  : next_state = {DEC_RET,  MEM_OFSP,  IPIPE_EN12};
            I1_RMW  : next_state = {DEC_RMWDD1,  MEM_IFPP1,  IPIPE_ENNEN2};
            I1_RST  : next_state = {DEC_NNCALL2,  MEM_OSSP_PCM2, IPIPE_NOP};  
            I1_R2R  : next_state = {DEC_EXEC, MEM_IFPP1, IPIPE_EN12A2}; 
            I1_JMPR : next_state = {DEC_N,    MEM_NOP,   IPIPE_ENNEN2A2};
            I1_HALT : next_state = {DEC_HALT, MEM_NOP  , IPIPE_EN2};
            default : next_state = {DEC_EXEC, MEM_IFPP1, IPIPE_EN12A2}; 
            endcase
        DEC_ED: 
            if (ed_nn)            next_state = {DEC_EDNN1,  MEM_IFPP1,   IPIPE_ENNEN2};
            
            
            else if (ed_rmw )     next_state = {DEC_RMW,    MEM_OF1,     IPIPE_EN12}; 
            else if (ed_blk_cp )  next_state = {DEC_EDBCP1, MEM_OFHL_PM, IPIPE_EN12};
            else if (ed_blk_in )  next_state = {DEC_EDBIN1, MEM_IOF_C,   IPIPE_EN12};
            else if (ed_blk_out)  next_state = {DEC_EDBOUT1,MEM_OFHL_PM, IPIPE_EN12};
            else if (ed_blk_mv )  next_state = {DEC_EDBMV1, MEM_OFHL_PM, IPIPE_EN12};
            else if (ed_retn   )  next_state = {DEC_RET,    MEM_OFSP,    IPIPE_EN12};
            else if (ed_in_reg )  next_state = {DEC_EDRD2,  MEM_IOF_C,   IPIPE_EN12};
            else if (ed_out_reg ) next_state = {DEC_IF2A,   MEM_IOS_C,   IPIPE_EN12};
            else                  next_state = {DEC_EXEC, MEM_IFPP1,    IPIPE_EN12A2};
                   
        DEC_EDNN1:                next_state = {DEC_EDNN2, MEM_IFPP1,     IPIPE_ENN}; 
        DEC_EDNN2: 
            if (ed_dbl_rd)      next_state = {DEC_EDRD1, MEM_OFNN,    IPIPE_EN12};    
            else                next_state = {DEC_EDWR,  MEM_OSNN,    IPIPE_EN12};
        DEC_EDRD1:              next_state = {DEC_EDRD2, MEM_OFADRP1,  IPIPE_ENN};  
        DEC_EDRD2:              next_state = {DEC_EXEC,   MEM_IFPP1,   IPIPE_ENNA2}; 
        DEC_EDWR:               next_state = {DEC_IF2A,   MEM_OSADRP1,  IPIPE_NOP};
        
        
        
        
        
        
        DEC_EDBCP1: 
            if (blk_cpi_done) next_state = {DEC_EXEC, MEM_IFPP1,IPIPE_ENNA2};
            else if(wb_int)   next_state = {DEC_INT1, MEM_NOP, IPIPE_ENNA2};
            else              next_state = {DEC_EDBCP2, MEM_NOP,  IPIPE_ENNA2};
        DEC_EDBCP2:                 next_state = {DEC_EDBCP3, MEM_NOP,     IPIPE_NOP};
        DEC_EDBCP3: if (fr[6])      next_state = {DEC_EXEC  , MEM_IFPP1,   IPIPE_NOP};
                    else            next_state = {DEC_EDBCP1, MEM_OFHL_PM, IPIPE_NOP};
                              
        DEC_EDBIN1:                  next_state = {DEC_EDBIN2, MEM_NOP,   IPIPE_ENN};
        DEC_EDBIN2: if (blk_done)    next_state = {DEC_IF2A,  MEM_OSHL_PM,IPIPE_NOP}; 
                    else if (wb_int) next_state = {DEC_INT1,  MEM_OSHL_PM,IPIPE_NOP};
                    else             next_state = {DEC_EDBIN3,MEM_OSHL_PM,IPIPE_NOP};
        DEC_EDBIN3:                  next_state = {DEC_EDBIN1, MEM_IOF_C,   IPIPE_NOP};
        
        DEC_EDBOUT1:                 next_state = {DEC_EDBOUT2, MEM_NOP,   IPIPE_ENN};                    
        DEC_EDBOUT2:if (blk_done)    next_state = {DEC_IF2A,  MEM_IOS_C,IPIPE_NOP};                    
                    else if (wb_int) next_state = {DEC_INT1,  MEM_IOS_C,IPIPE_NOP}; 
                    else             next_state = {DEC_EDBOUT3,MEM_IOS_C,IPIPE_NOP};        
        
        DEC_EDBOUT3:                 next_state = {DEC_EDBOUT1,MEM_OFHL_PM, IPIPE_NOP};

        DEC_EDBMV1:                  next_state = {DEC_EDBMV2, MEM_NOP,   IPIPE_ENN};                    
        DEC_EDBMV2: if (blk_done)    next_state = {DEC_IF2A,  MEM_OSDE_PM,IPIPE_NOP};                    
                    else if (wb_int) next_state = {DEC_INT1,  MEM_OSDE_PM,IPIPE_NOP}; 
                    else             next_state = {DEC_EDBMV3,MEM_OSDE_PM,IPIPE_NOP};        
        
        DEC_EDBMV3:                  next_state = {DEC_EDBMV1,MEM_OFHL_PM, IPIPE_NOP};
        
        DEC_N:
            if (INsA_6N7== ir1)      next_state = {DEC_NIN,  MEM_IOF_N, IPIPE_EN12};
            else if (OUTs6N7_A==ir1) next_state = {DEC_IF2A,  MEM_IOS_N, IPIPE_EN1}; 
            else if (LDs6HL7_N==ir1) next_state = {DEC_IF2A,  MEM_OS_HL_N, IPIPE_EN12};
            else if (jmpr_true)      next_state = {DEC_IF1,  MEM_REL2PC, IPIPE_NOP}; 
            else if (jmpr)           next_state = {DEC_IF2,  MEM_IFPP1,  IPIPE_NOP};
            else                     next_state = {DEC_EXEC, MEM_IFPP1,  IPIPE_EN12A2};
        DEC_DDN:
            if (INsA_6N7== ir1)      next_state = {DEC_NIN,  MEM_IOF_N, IPIPE_EN12};
            else if (OUTs6N7_A==ir1) next_state = {DEC_IF2A,  MEM_IOS_N, IPIPE_EN1}; 
            else if (LDs6HL7_N==ir1) next_state = {DEC_IF1,  MEM_OSIXpD, IPIPE_ENN};
            else                     next_state = {DEC_EXEC, MEM_IFPP1,  IPIPE_EN12A2};
        
        DEC_NIN:                     next_state = {DEC_IF2,  MEM_IFPP1,    IPIPE_ENNA2};
        
        
        
        
        
        
        
        DEC_NN:
            if      (callnn_true)     next_state = {DEC_NNCALL1, MEM_NOP, IPIPE_ENN};
                                                                                     
                                                                                     
            
            else if (jmpnn_true)      next_state = {DEC_NNJMP,  MEM_NOP,  IPIPE_ENN};    
                                                                                         
            else if (LDs6NN7_A==ir1)  next_state = {DEC_NNOS3,   MEM_IFPP1,  IPIPE_ENN};
            else if (LDs6NN7_HL==ir1) next_state = {DEC_NNOS1,   MEM_IFPP1,  IPIPE_ENN};
            else if (LDsA_6NN7==ir1)  next_state = {DEC_NNOF3,    MEM_IFPP1,  IPIPE_ENN};
            else if (LDsHL_6NN7==ir1) next_state = {DEC_NNOF1,    MEM_IFPP1,  IPIPE_ENN};
            else                      next_state = { DEC_IF2, MEM_IFPP1, IPIPE_ENNEN2A2};
        
        DEC_NNCALL1:        next_state = {DEC_NNCALL2, MEM_CALL ,  IPIPE_NOP}; 
        DEC_NNCALL2:        next_state = {DEC_IF1,    MEM_OSSP,   IPIPE_NOP};
        DEC_NNJMP:        next_state = {DEC_IF2,     MEM_IFNN  , IPIPE_NOP};
        
        
        
        
        DEC_NNOS1:           next_state = {DEC_NNOS2,   MEM_OSNN,   IPIPE_EN1};
        DEC_NNOS2:           next_state = {DEC_IF2A,    MEM_OSADRP1,   IPIPE_NOP};
        DEC_NNOS3:           next_state = {DEC_IF2A,    MEM_OSNN,   IPIPE_EN1};
        
        DEC_NNOF1:           next_state = {DEC_NNOF2,  MEM_OFNN, IPIPE_EN12};
        DEC_NNOF2:           next_state = {DEC_NNOF4,  MEM_OFADRP1, IPIPE_ENN};
        DEC_NNOF3:           next_state = {DEC_NNOF4,  MEM_OFNN, IPIPE_EN12};
        DEC_NNOF4: if (ex_tos_hl) next_state = {DEC_EXSPHL,   MEM_NOP, IPIPE_ENNA2};          
                   else           next_state = {DEC_EXEC,   MEM_IFPP1, IPIPE_ENNA2};
        
        DEC_DDOS:            next_state = {DEC_IF2A, MEM_OSIXpD, IPIPE_EN12};
        DEC_DDOF:            next_state = {DEC_OF  , MEM_OFIXpD,  IPIPE_EN12};
        
        
        DEC_OF:              next_state = {DEC_EXEC,  MEM_IFPP1 , IPIPE_ENNA2};
        DEC_POP:             next_state = {DEC_NNOF4,  MEM_OFSP, IPIPE_ENN };
        DEC_PUSH:            next_state = {DEC_IF2A ,  MEM_OSSP, IPIPE_NOP };
        
        
        DEC_RET:             next_state = { DEC_RET2, MEM_OFSP, IPIPE_ENN };
        DEC_RET2:            next_state = { DEC_NNJMP, MEM_NOP, IPIPE_ENN };
                                                                 
        DEC_RMW:             next_state = {DEC_RMW2,  MEM_NOP,   IPIPE_ENNA2}; 
        DEC_RMW2:            next_state = {DEC_IF2A ,  MEM_OSADR, IPIPE_NOP }; 
        
        
        
        DEC_CBM: if (CB_BIT==ir1[9:6]) next_state = {DEC_IF2, MEM_IFPP1,   IPIPE_ENNA2};
                 else                 next_state = {DEC_RMW2 ,  MEM_NOP,  IPIPE_ENNA2};
        
        
        
        
        
        
        
        
        DEC_PFxCB:     next_state = { DEC_PFxCB2, MEM_IFPP1,  IPIPE_EN1}; 
        DEC_PFxCB2:    next_state = { DEC_PFxCB3, MEM_OFIXpD, IPIPE_EN12}; 
        DEC_PFxCB3:    next_state = { DEC_PFxCB4, MEM_NOP,    IPIPE_ENNA2}; 
        DEC_PFxCB4:    next_state = { DEC_IF2A,   MEM_OSADR,  IPIPE_NOP};  
        
        
        
        
        
        
        
        DEC_INT1:       next_state = {DEC_INT2, MEM_OSSP_PCM2, IPIPE_NOP};   
        DEC_INT2:       next_state = {DEC_INT3, MEM_OSSP,   IPIPE_NOP};      
        DEC_INT3:       next_state = {DEC_INT4, MEM_INTA,     IPIPE_NOP};
        DEC_INT4:       next_state = {DEC_INT5, MEM_NOP,      IPIPE_ENN};
        DEC_INT5:       next_state = {DEC_INT6,  MEM_IFINT,  IPIPE_NOP}; 
        DEC_INT6:       next_state = {DEC_RET2, MEM_IFPP1,   IPIPE_ENN};
        DEC_EXSPHL:     next_state = {DEC_PUSH, MEM_OSSP,     IPIPE_NOP};
        DEC_RMWDD1:     next_state = {DEC_RMW,  MEM_OFIXpD,   IPIPE_EN1};
        default:        next_state = {DEC_IDLE, MEM_NOP,      IPIPE_NOP};
    endcase
end


always @(posedge wb_clk_i or posedge rst_i)
    if (rst_i) dec_state <= DEC_IDLE;
    else   if (wb_rdy_nhz )   dec_state <= next_dec_state;





wire update_prefix = dec_state == DEC_EXEC  | dec_state == DEC_DDFD | dec_state == DEC_PFxCB;
wire iext_ed =  update_prefix & (ir1[7:0]==8'hed);
wire iext_cb =  update_prefix & (ir1[7:0]==8'hcb);
always @(posedge wb_clk_i or posedge rst_i)
    if (rst_i) ir1 <=   NOP;
    else if (wb_rdy_nhz & next_pipe_state[3]) ir1 <=  {iext_ed, iext_cb, wb_dat_i} ;





always @(posedge wb_clk_i)
    if  (wb_rdy_nhz & next_dec_state == DEC_EXEC) {ir1dd, ir1fd } <= 2'b0;
    else if ( wb_rdy_nhz & update_prefix )
        {ir1dd, ir1fd } <= {  (ir1[7:0]==8'hdd ) | ir1dd & (ir1[7:0]!=8'hed) & (ir1[7:0]!=8'hfd),
                              (ir1[7:0]==8'hfd ) | ir1fd & (ir1[7:0]!=8'hed) & (ir1[7:0]!=8'hdd) };                 
                                           









always @(posedge wb_clk_i or posedge rst_i)
    if (rst_i) ir2 <= 10'h0;
    else if (wb_rdy_nhz & next_pipe_state[2]) ir2 <= ir1;
    
wire kill_prefix = next_mem_state == MEM_OFIXpD;
always @(posedge wb_clk_i or posedge rst_i)
    if (rst_i)
    begin
        ir2dd <= 1'b0;
        ir2fd <= 1'b0;
    end
    else if (wb_rdy_nhz & next_pipe_state[2])
    begin
        ir2dd <= ir1dd & ~kill_prefix;
        ir2fd <= ir1fd & ~kill_prefix;
    end
    
always @(posedge wb_clk_i )
    if (wb_rdy_nhz & next_pipe_state[0]) exec_ir2 <= 1'b1;
    else                                 exec_ir2 <= 1'b0;





always @(posedge wb_clk_i)
    if ((dec_state == DEC_EXEC) | (dec_state == DEC_DDFD))   
                            ex_tos_hl <= (ir1 == EXs6SP7_HL);





always @(posedge wb_clk_i)
    if (dec_state == DEC_ED)           blk_inc_flg <= dec_blk_inc;
    else if (dec_state == DEC_EXEC)    blk_inc_flg <= 1'b0;
always @(posedge wb_clk_i)
    if (dec_state == DEC_ED)           blk_rpt_flg <= dec_blk_rpt;
    else if (dec_state == DEC_EXEC)    blk_rpt_flg <= 1'b0;
    else if (dec_state == DEC_INT1)    blk_rpt_flg <= 1'b0;

always @(posedge wb_clk_i)
    if (dec_state == DEC_ED)           blk_io_flg <= dec_blk_io;
    else if (dec_state == DEC_EXEC)    blk_io_flg <= 1'b0;






always @(posedge wb_clk_i) if (wb_rdy_nhz) wb_adr_o <= mux21;



always @(posedge wb_clk_i or posedge rst_i)
    if (rst_i)         wb_we_o <= 1'b0;
    else if (wb_rdy_nhz) wb_we_o <= we_next; 
    
    
    



wire no_mem_start =  (next_mem_state == MEM_NOP) | (next_mem_state == MEM_REL2PC);
always @(posedge wb_clk_i or posedge rst_i)
    if (rst_i)         wb_cyc_o <= 1'b0;
    else if (wb_rdy_nhz) wb_cyc_o <= ~no_mem_start;



always @(posedge wb_clk_i or posedge rst_i)
    if (rst_i)         wb_stb_o <= 1'b0;
    else if (wb_rdy_nhz) wb_stb_o <= ~no_mem_start  ;





always @(posedge wb_clk_i or posedge rst_i)
    if (rst_i)         wb_tga_o <= 2'b0;
    else if (wb_rdy_nhz)
    begin
        if (next_mem_state == MEM_IOF_C |   
            next_mem_state == MEM_IOS_C |
            next_mem_state == MEM_IOF_N |
            next_mem_state == MEM_IOS_N     ) wb_tga_o <= TAG_IO;

        else if (next_mem_state == MEM_INTA ) wb_tga_o <= TAG_INT;
        else                                  wb_tga_o <= 2'b0   ;
    end































always @(posedge wb_clk_i or posedge rst_i)
    if       (rst_i)                                 flag_os1 <= 1'b0;
    else if  ((DEC_EXEC == next_dec_state) & wb_rdy) flag_os1 <= 1'b0;
    else if  ( we_next & wb_rdy_nhz  )               flag_os1 <= 1'b1;

wire ir2_cb_shift =  (ir2[9:6] == 4'b01_00) ; 
wire ir2_cb_bit   =  (ir2[9:6] ==  CB_RES ) |
                     (ir2[9:6] ==  CB_SET )  ;

wire [15:0] dec_const  = (DEC_INT1 == dec_state) & blk_rpt_flg ? 16'h3 :
                         (DEC_INT1 == dec_state)               ? 16'h2 :
                                                                 16'h1  ;
                                                          
wire [15:0] pc_123 = pc -  dec_const;




always @(posedge wb_clk_i or posedge rst_i)
    if (rst_i)  nn <= 6'h00;
    else if (wb_rdy_nhz)
    begin
        
        
        if ( we_next & flag_os1 & ~blk_rpt_flg)              nn <= { nn[7:0], nn[15:8] } ;
        
        else if( next_mem_state == MEM_CALL)                 nn <= {pc};
        else if( next_mem_state == MEM_OSSP_PCM2)            nn <= {pc_123};
        else if(EXs6SP7_HL== ir2 & ir2dd & exec_ir2)         nn <= ixr;
        else if(EXs6SP7_HL== ir2 & ir2fd & exec_ir2)         nn <= iyr;
        else if(EXs6SP7_HL== ir2         & exec_ir2)          nn <= hl;
        else if((INCs6HL7==ir2 | DECs6HL7==ir2) & exec_ir2)   nn[15:8] <= alu8_out;
        else if( ir2_cb_shift & MEM_OSADR == next_mem_state ) nn[15:8] <= sh_alu;
        else if( ir2_cb_bit   & MEM_OSADR == next_mem_state ) nn[15:8] <= bit_alu;
        else if( ED_RRD == ir2 & MEM_OSADR == next_mem_state) nn[15:8] <= {ar[3:0], nn[15:12]};
        else if( ED_RLD == ir2 & MEM_OSADR == next_mem_state) nn[15:8] <= {nn[11:8], ar[3:0] };
        
        else if (next_pipe_state[1])  nn  <= { wb_dat_i, nn[15:8] };   
        
        
        else if (we_next & ir1 == PUSHsHL)  nn   <= hl_or_ixiy;   
        else if(we_next & ( next_mem_state == MEM_OS1     |
                            next_mem_state == MEM_OSIXpD  |
                            next_mem_state == MEM_OSSP    |
                            next_mem_state == MEM_IOS_N   |
                            next_mem_state == MEM_IOS_C   |
                            next_mem_state == MEM_OSNN     ) )
            
            
            
            begin
                 if (os_a)     nn       <= {ar, fr };  
                 if (os_b)     nn       <= {br, cr };  
                 if (os_c)     nn[15:8] <= cr;  
                 if (os_d)     nn       <= {dr, er };  
                 if (os_e)     nn[15:8] <=  er;  
                 if (os_h)     nn       <= {hr, lr };  
                 if (os_l)     nn[15:8] <= lr;  
                 if (os_bc)    nn       <= {cr, br };  
                 if (os_de)    nn       <= {er, dr };
                 if (os_sp)    nn       <= {sp[7:0], sp[15:8] };
                 if (os_hl)    nn       <= {lr, hr };
                 if (os_ixr)   nn       <= {ixr[7:0], ixr[15:8] };
                 if (os_iyr)   nn       <= {iyr[7:0], iyr[15:8] };            
            end
        
    end
          



always @(posedge wb_clk_i or posedge rst_i)
    if (rst_i)   pc <= 16'h0;
    else if (wb_rdy_nhz)
    begin
        if (next_mem_state == MEM_DECPC) pc <= pc - 16'h1;  
        if (next_mem_state == MEM_IFPP1) pc <= adr_alu;
        if (next_mem_state == MEM_CALL ) pc <= nn;         
        if (next_mem_state == MEM_RST) pc <= adr_alu;
        if (next_mem_state == MEM_JMPHL) pc <= adr_alu;
        if (next_mem_state == MEM_OSSP_PCM2) pc <= { 10'h0, ir1[5:3], 3'h0} ;
        if (next_mem_state == MEM_IFNN ) pc <= adr_alu;    
        if (next_mem_state == MEM_REL2PC) pc <= adr_alu;
        if (next_mem_state == MEM_IFINT) pc <= adr_alu;  
    end




















wire ed_ld_spreg = (ED_LDsREG_6NN7 == {ir2[9:6],ir2[3:0]}) & (ir2[5:4] == DBL_REG_SP);

always @(posedge wb_clk_i )
begin
    if (exec_ir2 )   
        begin  
            if (LDsSP_NN   == ir2)   sp <= nn;
            if (ed_ld_spreg)         sp <= nn;
            if (  DECsSP   == ir2 )  sp <= add16;
            if (  INCsSP   == ir2 )  sp <= add16;
        end
    if (wb_rdy_nhz)  
    begin
         if (  LDsSP_HL == ir1 & dec_state == DEC_EXEC)  sp <= hl_or_ixiy;
         if (  LDsSP_HL == ir1 & dec_state == DEC_DDFD)  sp <= hl_or_ixiy;
         if (next_mem_state == MEM_OFSP      ) sp <= adr_alu;
         if (next_mem_state == MEM_OSSP      ) sp <= adr_alu;
         if (next_mem_state == MEM_OSSP_PCM2 ) sp <= adr_alu;
         
         if (next_mem_state == MEM_CALL      ) sp <= adr_alu;
    end
end




always @(posedge wb_clk_i or posedge rst_i)
    if (rst_i) inst_haz <= 1'b0;
    else if  (we_next & (pc - 16'h1) == mux21)  inst_haz <= 1'b1;
    else if  (dec_state == DEC_EXEC)  inst_haz <= 1'b0;   












always @(posedge wb_clk_i or posedge rst_i)
    if (rst_i)                   int_en <= 1'b0;
    else if (wb_rdy_nhz)
    begin
        if      ((dec_state == DEC_EXEC) & (DI== ir1))  int_en <= 1'b0;
        else if ((dec_state == DEC_EXEC) & en_int_next) int_en <= 1'b1; 
        if      (dec_state == DEC_INT1)                 int_en <= 1'b0;
    end


always @(posedge wb_clk_i or posedge rst_i)
    if (rst_i)                                      en_int_next <=1'b0;
    else if (wb_rdy_nhz)
    begin
        if ((dec_state == DEC_EXEC) & (EI== ir1))       en_int_next <=1'b1;
        else if ((dec_state == DEC_RET ) & (ED_RETI == ir2)) en_int_next <=1'b1;
        else if (dec_state == DEC_EXEC)                 en_int_next <=1'b0;
    end
    
always @(posedge wb_clk_i)  
    wb_irq_sync <= int_req_i;

assign  wb_int = wb_irq_sync & int_en;

endmodule    
             

`line 1 "F:\verilog\wb_z80\new_wb_z80\wb_z80\rtl\z80_sram.v"














































































module z80_sram(
    
    clk, rst, ce, we, oe, addr, di, do
);

    
    
    
    parameter aw = 15; 
    parameter dw = 8; 

    
    
    



    output [dw-1:0] do;   


    input           clk;  
    input           rst;  
    input           ce;   
    input           we;   
    input           oe;   
    input  [aw-1:0] addr; 
    input  [dw-1:0] di;   







reg  [dw-1:0] mem [(1<<aw)-1:0];    







assign do =  mem[addr];




always@(posedge clk)
    if (ce && we)
        mem[addr] <=  di;





	parameter init_file       = "z80_test2.ihx";
  reg [8*256 : 1] ram_initf;
	 initial begin
	  $convert_hex2ver(init_file, 8, ram_initf);
        $readmemh(ram_initf, mem);
	 $display("Initial Program Loaded. %s",ram_initf);
	 $display("Several Portions are modified from original. Grep and see TAK.");
	$display(" RAM 32k->64K ");				
	 end



      
      
      
      
 reg [0:639] char_buffer;
	task Cprint;
		integer i;
		begin :Block
			i=0;
			while (1) begin
				if (char_buffer[i*8 +:8] ===8'h00 || char_buffer[i*8 +:8]===8'hxx) begin
						disable Block;
				end	
				$write("%c",char_buffer[i*8 +:8]);
				i=i+1;  
			end
		end
	endtask

  
   integer  counter=0;    
   always @ (posedge clk ) begin
	 if (ce && we) begin 
             
                     
           	   if (addr== 16'h83e8) begin
				if (di===8'h00) begin
					
				      char_buffer[counter*8 +:8]=di;
					if (char_buffer[0  +:8*7]==="$finish") begin
							$stop;				
							
					end else if (char_buffer[0  +:8*5]==="$time") begin
							$display("Current Time on Simulation=%d",$time);				
							
					end else  Cprint;

					for (counter=0; counter< 80; counter=counter+1) begin
						char_buffer[counter*8 +:8]=8'h00;
					end
					counter=0;
				end else begin
					char_buffer[counter*8 +:8]=di;
					counter=counter+1;
				end
		   end
       	   else if (addr== 16'h83e9) begin
				$write("%h ",di);
		   end else if (addr== 16'h83eb) begin
				 $write("%h ",{di,mem[addr-1]});
		   end else if (addr== 16'h83ef) begin
				 $write("%h ",{di,mem[addr-1],mem[addr-2],mem[addr-3]});
		   end 
	end
   end 

endmodule



